@@ -2,8 +2,8 @@
 
   error.c -
 
-  $Author: nobu $
-  $Date: 2003/07/30 06:10:08 $
+  $Author: dave $
+  $Date: 2003/12/21 07:28:53 $
   created at: Mon Aug  9 16:11:34 JST 1993
 
   Copyright (C) 1993-2003 Yukihiro Matsumoto
@@ -111,10 +111,12 @@ warn_print(fmt, args)
     va_list args;
 {
     char buf[BUFSIZ];
+    int len;
 
     err_snprintf(buf, BUFSIZ, fmt, args);
-    rb_write_error(buf);
-    rb_write_error("\n");
+    len = strlen(buf);
+    buf[len++] = '\n';
+    rb_write_error2(buf, len);
 }
 
 void
@@ -164,9 +166,11 @@ static VALUE
 rb_warn_m(self, mesg)
     VALUE self, mesg;
 {
-    rb_io_write(rb_stderr, mesg);
-    rb_io_write(rb_stderr, rb_default_rs);
-    return mesg;
+    if (!NIL_P(ruby_verbose)) {
+	rb_io_write(rb_stderr, mesg);
+	rb_io_write(rb_stderr, rb_default_rs);
+    }
+    return Qnil;
 }
 
 void
@@ -180,16 +184,18 @@ rb_bug(fmt, va_alist)
 {
     char buf[BUFSIZ];
     va_list args;
+    FILE *out = stderr;
+    int len = err_position(buf, BUFSIZ);
 
-    snprintf(buf, BUFSIZ, "[BUG] %s", fmt);
-    ruby_in_eval = 0;
-
-    va_init_list(args, fmt);
-    warn_print(buf, args);
-    va_end(args);
-    snprintf(buf, BUFSIZ, "ruby %s (%s) [%s]\n", RUBY_VERSION, RUBY_RELEASE_DATE, RUBY_PLATFORM);
-    rb_write_error(buf);
-    rb_write_error("\n");
+    if (fwrite(buf, 1, len, out) == len ||
+	fwrite(buf, 1, len, (out = stdout)) == len) {
+	fputs("[BUG] ", out);
+	va_init_list(args, fmt);
+	vfprintf(out, fmt, args);
+	va_end(args);
+	fprintf(out, "\nruby %s (%s) [%s]\n\n",
+		RUBY_VERSION, RUBY_RELEASE_DATE, RUBY_PLATFORM);
+    }
     abort();
 }
 
@@ -312,23 +318,43 @@ rb_exc_new3(etype, str)
     return rb_exc_new(etype, RSTRING(str)->ptr, RSTRING(str)->len);
 }
 
+/*
+ * call-seq:
+ *    Exception.new(msg = nil)   =>  exception
+ *
+ *  Construct a new Exception object, optionally passing in 
+ *  a message.
+ */
+
 static VALUE
 exc_initialize(argc, argv, exc)
     int argc;
     VALUE *argv;
     VALUE exc;
 {
-    VALUE mesg;
+    VALUE arg;
 
-    if (rb_scan_args(argc, argv, "01", &mesg) == 1) {
+    if (rb_scan_args(argc, argv, "01", &arg) == 1) {
+	VALUE mesg = arg;
 	StringValue(mesg);	/* ensure mesg can be converted to String */
     }
-    rb_iv_set(exc, "mesg", mesg);
+    rb_iv_set(exc, "mesg", arg);
     rb_iv_set(exc, "bt", Qnil);
 
     return exc;
 }
 
+/*
+ *  call-seq:
+ *     exc.exception(string) -> an_exception or exc
+ *  
+ *  With no argument, or if the argument is the same as the receiver,
+ *  return the receiver. Otherwise, create a new
+ *  exception object of the same class as the receiver, but with a
+ *  message equal to <code>string.to_str</code>.
+ *     
+ */
+
 static VALUE
 exc_exception(argc, argv, self)
     int argc;
@@ -345,6 +371,14 @@ exc_exception(argc, argv, self)
     return exc;
 }
 
+/*
+ * call-seq:
+ *   exception.to_s   =>  string
+ *
+ * Returns exception's message (or the name of the exception if
+ * no message is set).
+ */
+
 static VALUE
 exc_to_s(exc)
     VALUE exc;
@@ -356,6 +390,31 @@ exc_to_s(exc)
     return mesg;
 }
 
+/*
+ * call-seq:
+ *   exception.message   =>  string
+ *   exception.to_str    =>  string
+ *
+ * Returns the result of invoking <code>exception.to_s</code>.
+ * Normally this returns the exception's message or name. By
+ * supplying a to_str method, exceptions are agreeing to
+ * be used where Strings are expected.
+ */
+
+static VALUE
+exc_to_str(exc)
+    VALUE exc;
+{
+    return rb_funcall(exc, rb_intern("to_s"), 0, 0);
+}
+
+/*
+ * call-seq:
+ *   exception.inspect   => string
+ *
+ * Return this exception's class name an message
+ */
+
 static VALUE
 exc_inspect(exc)
     VALUE exc;
@@ -378,6 +437,35 @@ exc_inspect(exc)
     return str;
 }
 
+/*
+ *  call-seq:
+ *     exception.backtrace    => array
+ *  
+ *  Returns any backtrace associated with the exception. The backtrace
+ *  is an array of strings, each containing either ``filename:lineNo: in
+ *  `method''' or ``filename:lineNo.''
+ *     
+ *     def a
+ *       raise "boom"
+ *     end
+ *     
+ *     def b
+ *       a()
+ *     end
+ *     
+ *     begin
+ *       b()
+ *     rescue => detail
+ *       print detail.backtrace.join("\n")
+ *     end
+ *     
+ *  <em>produces:</em>
+ *     
+ *     prog.rb:2:in `a'
+ *     prog.rb:6:in `b'
+ *     prog.rb:10
+*/
+
 static VALUE
 exc_backtrace(exc)
     VALUE exc;
@@ -411,6 +499,16 @@ check_backtrace(bt)
     return bt;
 }
 
+/*
+ *  call-seq:
+ *     exc.set_backtrace(array_   =>  array
+ *  
+ *  Sets the backtrace information associated with <i>exc</i>. The
+ *  argument must be an array of <code>String</code> objects in the
+ *  format described in <code>Exception#backtrace</code>.
+ *     
+ */
+
 static VALUE
 exc_set_backtrace(exc, bt)
     VALUE exc;
@@ -472,7 +570,9 @@ name_err_initialize(argc, argv, self)
     VALUE *argv;
     VALUE self;
 {
-    VALUE name = (argc > 1) ? argv[--argc] : Qnil;
+    VALUE name;
+
+    name = (argc > 1) ? argv[--argc] : Qnil;
     exc_initialize(argc, argv, self);
     rb_iv_set(self, "name", name);
     return self;
@@ -513,6 +613,37 @@ rb_invalid_str(str, type)
     rb_raise(rb_eArgError, "invalid value for %s: %s", type, RSTRING(s)->ptr);
 }
 
+/* 
+ *  Document-module: Errno
+ *
+ *  Ruby exception objects are subclasses of <code>Exception</code>.
+ *  However, operating systems typically report errors using plain
+ *  integers. Module <code>Errno</code> is created dynamically to map
+ *  these operating system errors to Ruby classes, with each error
+ *  number generating its own subclass of <code>SystemCallError</code>.
+ *  As the subclass is created in module <code>Errno</code>, its name
+ *  will start <code>Errno::</code>.
+ *     
+ *  The names of the <code>Errno::</code> classes depend on
+ *  the environment in which Ruby runs. On a typical Unix or Windows
+ *  platform, there are <code>Errno</code> classes such as
+ *  <code>Errno::EACCES</code>, <code>Errno::EAGAIN</code>,
+ *  <code>Errno::EINTR</code>, and so on.
+ *     
+ *  The integer operating system error number corresponding to a
+ *  particular error is available as the class constant
+ *  <code>Errno::</code><em>error</em><code>::Errno</code>.
+ *     
+ *     Errno::EACCES::Errno   #=> 13
+ *     Errno::EAGAIN::Errno   #=> 11
+ *     Errno::EINTR::Errno    #=> 4
+ *     
+ *  The full list of operating system errors on your particular platform
+ *  are available as the constants of <code>Errno</code>.
+ *
+ *     Errno.constants   #=> E2BIG, EACCES, EADDRINUSE, EADDRNOTAVAIL, ...
+ */
+
 static st_table *syserr_tbl;
 
 static VALUE
@@ -606,7 +737,7 @@ static VALUE
 syserr_eqq(self, exc)
     VALUE self, exc;
 {
-    VALUE num;
+    VALUE num, e;
 
     if (!rb_obj_is_kind_of(exc, rb_eSystemCallError)) return Qfalse;
     if (self == rb_eSystemCallError) return Qtrue;
@@ -620,12 +751,21 @@ syserr_eqq(self, exc)
 	}
 	num = rb_const_get(klass, rb_intern("Errno"));
     }
-    if (rb_const_get(self, rb_intern("Errno")) == num)
+    e = rb_const_get(self, rb_intern("Errno"));
+    if (FIXNUM_P(num) ? num == e : rb_equal(num, e))
 	return Qtrue;
     return Qfalse;
 }
 
-static void init_syserr _((void));
+/*
+ *  Descendents of class <code>Exception</code> are used to communicate
+ *  between <code>raise</code> methods and <code>rescue</code>
+ *  statements in <code>begin/end</code> blocks. <code>Exception</code>
+ *  objects carry information about the exception---its type (the
+ *  exception's class name), an optional descriptive string, and
+ *  optional traceback information. Programs may subclass 
+ *  <code>Exception</code> to add additional information.
+ */
 
 void
 Init_Exception()
@@ -635,8 +775,8 @@ Init_Exception()
     rb_define_method(rb_eException, "exception", exc_exception, -1);
     rb_define_method(rb_eException, "initialize", exc_initialize, -1);
     rb_define_method(rb_eException, "to_s", exc_to_s, 0);
-    rb_define_method(rb_eException, "to_str", exc_to_s, 0);
-    rb_define_method(rb_eException, "message", exc_to_s, 0);
+    rb_define_method(rb_eException, "to_str", exc_to_str, 0);
+    rb_define_method(rb_eException, "message", exc_to_str, 0);
     rb_define_method(rb_eException, "inspect", exc_inspect, 0);
     rb_define_method(rb_eException, "backtrace", exc_backtrace, 0);
     rb_define_method(rb_eException, "set_backtrace", exc_set_backtrace, 1);
@@ -670,7 +810,13 @@ Init_Exception()
     rb_eSecurityError = rb_define_class("SecurityError", rb_eStandardError);
     rb_eNoMemError = rb_define_class("NoMemoryError", rb_eException);
 
-    init_syserr();
+    syserr_tbl = st_init_numtable();
+    rb_eSystemCallError = rb_define_class("SystemCallError", rb_eStandardError);
+    rb_define_method(rb_eSystemCallError, "initialize", syserr_initialize, -1);
+    rb_define_method(rb_eSystemCallError, "errno", syserr_errno, 0);
+    rb_define_singleton_method(rb_eSystemCallError, "===", syserr_eqq, 1);
+
+    rb_mErrno = rb_define_module("Errno");
 
     rb_define_global_function("warn", rb_warn_m, 1);
 }
@@ -750,7 +896,7 @@ rb_sys_fail(mesg)
 
     errno = 0;
     if (n == 0) {
-	rb_bug("rb_sys_fail() - errno == 0");
+	rb_bug("rb_sys_fail(%s) - errno == 0", mesg ? mesg : "");
     }
 
     arg = mesg ? rb_str_new2(mesg) : Qnil;
@@ -804,16 +950,9 @@ rb_check_frozen(obj)
     if (OBJ_FROZEN(obj)) rb_error_frozen(rb_obj_classname(obj));
 }
 
-static void
-init_syserr()
+void
+Init_syserr()
 {
-    syserr_tbl = st_init_numtable();
-    rb_eSystemCallError = rb_define_class("SystemCallError", rb_eStandardError);
-    rb_define_method(rb_eSystemCallError, "initialize", syserr_initialize, -1);
-    rb_define_method(rb_eSystemCallError, "errno", syserr_errno, 0);
-    rb_define_singleton_method(rb_eSystemCallError, "===", syserr_eqq, 1);
-
-    rb_mErrno = rb_define_module("Errno");
 #ifdef EPERM
     set_syserr(EPERM, "EPERM");
 #endif
