@@ -2,14 +2,14 @@
 
   digest.c -
 
-  $Author: usa $
+  $Author: knu $
   created at: Fri May 25 08:57:27 JST 2001
 
   Copyright (C) 1995-2001 Yukihiro Matsumoto
   Copyright (C) 2001 Akinori MUSHA
 
   $RoughId: digest.c,v 1.16 2001/07/13 15:38:27 knu Exp $
-  $Id: digest.c,v 1.11 2002/12/21 18:02:01 usa Exp $
+  $Id: digest.c,v 1.14 2003/10/15 13:50:51 knu Exp $
 
 ************************************************/
 
@@ -73,7 +73,8 @@ rb_digest_base_alloc(klass)
 
     algo = get_digest_base_metadata(klass);
 
-    pctx = xmalloc(algo->ctx_size);
+    /* XXX: An uninitialized buffer leads ALGO_Equal() to fail */
+    pctx = xcalloc(algo->ctx_size, 1);
     algo->init_func(pctx);
 
     obj = Data_Wrap_Struct(klass, 0, free, pctx);
@@ -303,7 +304,7 @@ Init_digest()
     rb_define_singleton_method(cDigest_Base, "hexdigest", rb_digest_base_s_hexdigest, 1);
 
     rb_define_method(cDigest_Base, "initialize", rb_digest_base_init, -1);
-    rb_define_method(cDigest_Base, "copy_object",  rb_digest_base_copy, 1);
+    rb_define_method(cDigest_Base, "initialize_copy",  rb_digest_base_copy, 1);
     rb_define_method(cDigest_Base, "update", rb_digest_base_update, 1);
     rb_define_method(cDigest_Base, "<<", rb_digest_base_update, 1);
     rb_define_method(cDigest_Base, "digest", rb_digest_base_digest, 0);
