@@ -3,7 +3,7 @@
   gdbm.c -
 
   $Author: matz $
-  $Date: 2001/05/07 06:18:47 $
+  $Date: 2003/05/06 06:51:30 $
   modified at: Mon Jan 24 15:59:52 JST 1994
 
 ************************************************/
@@ -14,10 +14,16 @@
 #include <fcntl.h>
 #include <errno.h>
 
-static VALUE cGDBM, rb_eGDBMError;
+static VALUE rb_cGDBM, rb_eGDBMError, rb_eGDBMFatalError;
 
 #define MY_BLOCK_SIZE (2048)
-#define MY_FATAL_FUNC (0)
+#define MY_FATAL_FUNC rb_gdbm_fatal
+static void
+rb_gdbm_fatal(msg)
+    char *msg;
+{
+    rb_raise(rb_eGDBMFatalError, msg);
+}
 
 struct dbmdata {
     int  di_size;
@@ -30,11 +36,11 @@ closed_dbm()
     rb_raise(rb_eRuntimeError, "closed GDBM file");
 }
 
-#define GetDBM(obj, dbmp) {\
+#define GetDBM(obj, dbmp) do {\
     Data_Get_Struct(obj, struct dbmdata, dbmp);\
     if (dbmp == 0) closed_dbm();\
     if (dbmp->di_dbm == 0) closed_dbm();\
-}
+} while (0)
 
 static void
 free_dbm(dbmp)
@@ -46,17 +52,26 @@ free_dbm(dbmp)
     }
 }
 
-static VALUE fgdbm_close _((VALUE));
+static VALUE
+fgdbm_close(obj)
+    VALUE obj;
+{
+    struct dbmdata *dbmp;
+
+    GetDBM(obj, dbmp);
+    gdbm_close(dbmp->di_dbm);
+    dbmp->di_dbm = 0;
+
+    return Qnil;
+}
+
+static VALUE fgdbm_s_alloc _((VALUE));
 
 static VALUE
-fgdbm_s_new(argc, argv, klass)
-    int argc;
-    VALUE *argv;
+fgdbm_s_alloc(klass)
     VALUE klass;
 {
-    VALUE obj = Data_Wrap_Struct(klass, 0, free_dbm, 0);
-    rb_obj_call_init(obj, argc, argv);
-    return obj;
+    return Data_Wrap_Struct(klass, 0, free_dbm, 0);
 }
 
 static VALUE
@@ -83,8 +98,7 @@ fgdbm_initialize(argc, argv, obj)
     if (!NIL_P(vflags))
         flags = NUM2INT(vflags);
 
-    file = rb_str_to_str(file);
-    Check_SafeStr(file);
+    SafeStringValue(file);
 
     dbm = 0;
     if (mode >= 0)
@@ -109,6 +123,7 @@ fgdbm_initialize(argc, argv, obj)
     }
 
     dbmp = ALLOC(struct dbmdata);
+    free_dbm(DATA_PTR(obj));
     DATA_PTR(obj) = dbmp;
     dbmp->di_dbm = dbm;
     dbmp->di_size = -1;
@@ -135,37 +150,23 @@ fgdbm_s_open(argc, argv, klass)
     return obj;
 }
 
-static VALUE
-fgdbm_close(obj)
-    VALUE obj;
-{
-    struct dbmdata *dbmp;
-
-    GetDBM(obj, dbmp);
-    gdbm_close(dbmp->di_dbm);
-    dbmp->di_dbm = 0;
-
-    return Qnil;
-}
-
 static VALUE
 rb_gdbm_fetch(dbm, key)
     GDBM_FILE dbm;
     datum key;
 {
     datum val;
-    NEWOBJ(str, struct RString);
-    OBJSETUP(str, rb_cString, T_STRING);
+    VALUE str;
 
     val = gdbm_fetch(dbm, key);
     if (val.dptr == 0)
         return Qnil;
 
-    str->ptr = 0;
-    str->len = val.dsize;
-    str->orig = 0;
-    str->ptr = REALLOC_N(val.dptr,char,val.dsize+1);
-    str->ptr[str->len] = '\0';
+    str = rb_obj_alloc(rb_cString);
+    RSTRING(str)->len = val.dsize;
+    RSTRING(str)->aux.capa = val.dsize;
+    RSTRING(str)->ptr = REALLOC_N(val.dptr,char,val.dsize+1);
+    RSTRING(str)->ptr[val.dsize] = '\0';
 
     OBJ_TAINT(str);
     return (VALUE)str;
@@ -178,7 +179,7 @@ rb_gdbm_fetch2(dbm, keystr)
 {
     datum key;
 
-    keystr = rb_str_to_str(keystr);
+    StringValue(keystr);
     key.dptr = RSTRING(keystr)->ptr;
     key.dsize = RSTRING(keystr)->len;
 
@@ -202,21 +203,20 @@ rb_gdbm_firstkey(dbm)
     GDBM_FILE dbm;
 {
     datum key;
-    NEWOBJ(str, struct RString);
-    OBJSETUP(str, rb_cString, T_STRING);
+    VALUE str;
 
     key = gdbm_firstkey(dbm);
     if (key.dptr == 0)
         return Qnil;
 
-    str->ptr = 0;
-    str->len = key.dsize;
-    str->orig = 0;
-    str->ptr = REALLOC_N(key.dptr,char,key.dsize+1);
-    str->ptr[str->len] = '\0';
+    str = rb_obj_alloc(rb_cString);
+    RSTRING(str)->len = key.dsize;
+    RSTRING(str)->aux.capa = key.dsize;
+    RSTRING(str)->ptr = REALLOC_N(key.dptr,char,key.dsize+1);
+    RSTRING(str)->ptr[RSTRING(str)->len] = '\0';
 
     OBJ_TAINT(str);
-    return (VALUE)str;
+    return str;
 }
 
 static VALUE
@@ -225,8 +225,7 @@ rb_gdbm_nextkey(dbm, keystr)
     VALUE keystr;
 {
     datum key, key2;
-    NEWOBJ(str, struct RString);
-    OBJSETUP(str, rb_cString, T_STRING);
+    VALUE str;
 
     key.dptr = RSTRING(keystr)->ptr;
     key.dsize = RSTRING(keystr)->len;
@@ -234,35 +233,26 @@ rb_gdbm_nextkey(dbm, keystr)
     if (key2.dptr == 0)
         return Qnil;
 
-    str->ptr = 0;
-    str->len = key2.dsize;
-    str->orig = 0;
-    str->ptr = REALLOC_N(key2.dptr,char,key2.dsize+1);
-    str->ptr[str->len] = '\0';
+    str = rb_obj_alloc(rb_cString);
+    RSTRING(str)->len = key2.dsize;
+    RSTRING(str)->aux.capa = key2.dsize;
+    RSTRING(str)->ptr = REALLOC_N(key2.dptr,char,key2.dsize+1);
+    RSTRING(str)->ptr[RSTRING(str)->len] = '\0';
 
     OBJ_TAINT(str);
-    return (VALUE)str;
+    return str;
 }
 
 static VALUE
 fgdbm_fetch(obj, keystr, ifnone)
     VALUE obj, keystr, ifnone;
 {
-    datum key;
-    struct dbmdata *dbmp;
-    GDBM_FILE dbm;
     VALUE valstr;
 
-    keystr = rb_str_to_str(keystr);
-    key.dptr = RSTRING(keystr)->ptr;
-    key.dsize = RSTRING(keystr)->len;
-
-    GetDBM(obj, dbmp);
-    dbm = dbmp->di_dbm;
-    valstr = rb_gdbm_fetch(dbm, key);
+    valstr = rb_gdbm_fetch3(obj, keystr);
     if (NIL_P(valstr)) {
 	if (ifnone == Qnil && rb_block_given_p())
-	    return rb_yield(rb_tainted_str_new(key.dptr, key.dsize));
+	    return rb_yield(keystr);
 	return ifnone;
     }
     return valstr;
@@ -299,7 +289,7 @@ fgdbm_index(obj, valstr)
     GDBM_FILE dbm;
     VALUE keystr, valstr2;
 
-    valstr = rb_str_to_str(valstr);
+    StringValue(valstr);
     GetDBM(obj, dbmp);
     dbm = dbmp->di_dbm;
     for (keystr = rb_gdbm_firstkey(dbm); RTEST(keystr);
@@ -333,6 +323,61 @@ fgdbm_indexes(argc, argv, obj)
     return new;
 }
 
+static VALUE
+fgdbm_select(argc, argv, obj)
+    int argc;
+    VALUE *argv;
+    VALUE obj;
+{
+    VALUE new = rb_ary_new2(argc);
+    int i;
+
+    if (rb_block_given_p()) {
+        GDBM_FILE dbm;
+        struct dbmdata *dbmp;
+        VALUE keystr;
+
+	if (argc > 0) {
+	    rb_raise(rb_eArgError, "wrong number arguments(%d for 0)", argc);
+	}
+        GetDBM(obj, dbmp);
+        dbm = dbmp->di_dbm;
+
+        for (keystr = rb_gdbm_firstkey(dbm); RTEST(keystr);
+             keystr = rb_gdbm_nextkey(dbm, keystr)) {
+            VALUE assoc = rb_assoc_new(keystr, rb_gdbm_fetch2(dbm, keystr));
+
+            if (RTEST(rb_yield(assoc)))
+                rb_ary_push(new, assoc);
+        }
+    }
+    else {
+	rb_warn("GDBM#select(index..) is deprecated; use GDBM#values_at");
+
+        for (i=0; i<argc; i++) {
+            rb_ary_push(new, rb_gdbm_fetch3(obj, argv[i]));
+        }
+    }
+
+    return new;
+}
+
+static VALUE
+fgdbm_values_at(argc, argv, obj)
+    int argc;
+    VALUE *argv;
+    VALUE obj;
+{
+    VALUE new = rb_ary_new2(argc);
+    int i;
+
+    for (i=0; i<argc; i++) {
+        rb_ary_push(new, rb_gdbm_fetch3(obj, argv[i]));
+    }
+
+    return new;
+}
+
 static VALUE
 rb_gdbm_delete(obj, keystr)
     VALUE obj, keystr;
@@ -342,7 +387,7 @@ rb_gdbm_delete(obj, keystr)
     GDBM_FILE dbm;
 
     rb_secure(4);
-    keystr = rb_str_to_str(keystr);
+    StringValue(keystr);
     key.dptr = RSTRING(keystr)->ptr;
     key.dsize = RSTRING(keystr)->len;
 
@@ -367,31 +412,11 @@ static VALUE
 fgdbm_delete(obj, keystr)
     VALUE obj, keystr;
 {
-    datum key;
-    struct dbmdata *dbmp;
-    GDBM_FILE dbm;
+    VALUE valstr;
 
-    rb_secure(4);
-    keystr = rb_str_to_str(keystr);
-    key.dptr = RSTRING(keystr)->ptr;
-    key.dsize = RSTRING(keystr)->len;
-
-    GetDBM(obj, dbmp);
-    dbm = dbmp->di_dbm;
-
-    if (!gdbm_exists(dbm, key)) {
-	if (rb_block_given_p()) rb_yield(keystr);
-	return Qnil;
-    }
-
-    if (gdbm_delete(dbm, key)) {
-	dbmp->di_size = -1;
-	rb_raise(rb_eGDBMError, "%s", gdbm_strerror(gdbm_errno));
-    }
-    else if (dbmp->di_size >= 0) {
-	dbmp->di_size--;
-    }
-    return obj;
+    valstr = fgdbm_fetch(obj, keystr, Qnil);
+    rb_gdbm_delete(obj, keystr);
+    return valstr;
 }
 
 static VALUE
@@ -434,13 +459,11 @@ fgdbm_delete_if(obj)
          keystr = rb_gdbm_nextkey(dbm, keystr)) {
 
 	valstr = rb_gdbm_fetch2(dbm, keystr);
-        ret = rb_protect(rb_yield, rb_assoc_new(rb_str_dup(keystr), valstr), &status);
-        if (status != 0) goto delete;
+        ret = rb_protect(rb_yield, rb_assoc_new(keystr, valstr), &status);
+        if (status != 0) break;
 	if (RTEST(ret)) rb_ary_push(ary, keystr);
-	else dbmp->di_size++;
     }
 
- delete:
     for (i = 0; i < RARRAY(ary)->len; i++)
         rb_gdbm_delete(obj, RARRAY(ary)->ptr[i]);
     if (status) rb_jump_tag(status);
@@ -462,6 +485,15 @@ fgdbm_clear(obj)
     dbm = dbmp->di_dbm;
     dbmp->di_size = -1;
 
+#if 0
+    while (key = gdbm_firstkey(dbm), key.dptr) {
+	if (gdbm_delete(dbm, key)) {
+	    free(key.dptr);
+	    rb_raise(rb_eGDBMError, "%s", gdbm_strerror(gdbm_errno));
+	}
+	free(key.dptr); 
+    }
+#else
     while (key = gdbm_firstkey(dbm), key.dptr) {
         for (; key.dptr; key = nextkey) {
             nextkey = gdbm_nextkey(dbm, key);
@@ -473,6 +505,7 @@ fgdbm_clear(obj)
             free(key.dptr);
         }
     }
+#endif
     dbmp->di_size = 0;
 
     return obj;
@@ -498,6 +531,8 @@ fgdbm_invert(obj)
     return hash;
 }
 
+static VALUE each_pair _((VALUE));
+
 static VALUE
 each_pair(obj)
     VALUE obj;
@@ -545,11 +580,11 @@ fgdbm_store(obj, keystr, valstr)
     GDBM_FILE dbm;
 
     rb_secure(4);
-    keystr = rb_str_to_str(keystr);
+    StringValue(keystr);
     key.dptr = RSTRING(keystr)->ptr;
     key.dsize = RSTRING(keystr)->len;
 
-    valstr = rb_str_to_str(valstr);
+    StringValue(valstr);
     val.dptr = RSTRING(valstr)->ptr;
     val.dsize = RSTRING(valstr)->len;
 
@@ -591,7 +626,7 @@ static VALUE
 fgdbm_empty_p(obj)
     VALUE obj;
 {
-    datum key, nextkey;
+    datum key;
     struct dbmdata *dbmp;
     GDBM_FILE dbm;
 
@@ -644,7 +679,7 @@ fgdbm_each_key(obj)
     for (keystr = rb_gdbm_firstkey(dbm); RTEST(keystr);
          keystr = rb_gdbm_nextkey(dbm, keystr)) {
 
-        rb_yield(rb_str_dup(keystr));
+        rb_yield(keystr);
     }
     return obj;
 }
@@ -663,8 +698,7 @@ fgdbm_each_pair(obj)
     for (keystr = rb_gdbm_firstkey(dbm); RTEST(keystr);
          keystr = rb_gdbm_nextkey(dbm, keystr)) {
 
-        rb_yield(rb_assoc_new(rb_str_dup(keystr), 
-                              rb_gdbm_fetch2(dbm, keystr)));
+        rb_yield(rb_assoc_new(keystr, rb_gdbm_fetch2(dbm, keystr)));
     }
 
     return obj;
@@ -722,7 +756,7 @@ fgdbm_has_key(obj, keystr)
     struct dbmdata *dbmp;
     GDBM_FILE dbm;
 
-    keystr = rb_str_to_str(keystr);
+    StringValue(keystr);
     key.dptr = RSTRING(keystr)->ptr;
     key.dsize = RSTRING(keystr)->len;
 
@@ -741,7 +775,7 @@ fgdbm_has_value(obj, valstr)
     GDBM_FILE dbm;
     VALUE keystr, valstr2;
 
-    valstr = rb_str_to_str(valstr);
+    StringValue(valstr);
     GetDBM(obj, dbmp);
     dbm = dbmp->di_dbm;
     for (keystr = rb_gdbm_firstkey(dbm); RTEST(keystr);
@@ -774,8 +808,7 @@ fgdbm_to_a(obj)
     for (keystr = rb_gdbm_firstkey(dbm); RTEST(keystr);
          keystr = rb_gdbm_nextkey(dbm, keystr)) {
 
-        rb_ary_push(ary, rb_assoc_new(rb_str_dup(keystr),
-                                      rb_gdbm_fetch2(dbm, keystr)));
+        rb_ary_push(ary, rb_assoc_new(keystr, rb_gdbm_fetch2(dbm, keystr)));
     }
 
     return ary;
@@ -905,74 +938,77 @@ fgdbm_reject(obj)
 void
 Init_gdbm()
 {
-    cGDBM = rb_define_class("GDBM", rb_cObject);
+    rb_cGDBM = rb_define_class("GDBM", rb_cObject);
     rb_eGDBMError = rb_define_class("GDBMError", rb_eStandardError);
-    rb_include_module(cGDBM, rb_mEnumerable);
+    rb_eGDBMFatalError = rb_define_class("GDBMFatalError", rb_eException);
+    rb_include_module(rb_cGDBM, rb_mEnumerable);
 
-    rb_define_singleton_method(cGDBM, "new", fgdbm_s_new, -1);
-    rb_define_singleton_method(cGDBM, "open", fgdbm_s_open, -1);
+    rb_define_alloc_func(rb_cGDBM, fgdbm_s_alloc);
+    rb_define_singleton_method(rb_cGDBM, "open", fgdbm_s_open, -1);
 
-    rb_define_method(cGDBM, "initialize", fgdbm_initialize, -1);
-    rb_define_method(cGDBM, "close", fgdbm_close, 0);
-    rb_define_method(cGDBM, "[]", fgdbm_aref, 1);
-    rb_define_method(cGDBM, "fetch", fgdbm_fetch_m, -1);
-    rb_define_method(cGDBM, "[]=", fgdbm_store, 2);
-    rb_define_method(cGDBM, "store", fgdbm_store, 2);
-    rb_define_method(cGDBM, "index",  fgdbm_index, 1);
-    rb_define_method(cGDBM, "indexes",  fgdbm_indexes, -1);
-    rb_define_method(cGDBM, "indices",  fgdbm_indexes, -1);
-    rb_define_method(cGDBM, "length", fgdbm_length, 0);
-    rb_define_alias(cGDBM,  "size", "length");
-    rb_define_method(cGDBM, "empty?", fgdbm_empty_p, 0);
-    rb_define_method(cGDBM, "each", fgdbm_each_pair, 0);
-    rb_define_method(cGDBM, "each_value", fgdbm_each_value, 0);
-    rb_define_method(cGDBM, "each_key", fgdbm_each_key, 0);
-    rb_define_method(cGDBM, "each_pair", fgdbm_each_pair, 0);
-    rb_define_method(cGDBM, "keys", fgdbm_keys, 0);
-    rb_define_method(cGDBM, "values", fgdbm_values, 0);
-    rb_define_method(cGDBM, "shift", fgdbm_shift, 0);
-    rb_define_method(cGDBM, "delete", fgdbm_delete, 1);
-    rb_define_method(cGDBM, "delete_if", fgdbm_delete_if, 0);
-    rb_define_method(cGDBM, "reject!", fgdbm_delete_if, 0);
-    rb_define_method(cGDBM, "reject", fgdbm_reject, 0);
-    rb_define_method(cGDBM, "clear", fgdbm_clear, 0);
-    rb_define_method(cGDBM,"invert", fgdbm_invert, 0);
-    rb_define_method(cGDBM,"update", fgdbm_update, 1);
-    rb_define_method(cGDBM,"replace", fgdbm_replace, 1);
-    rb_define_method(cGDBM,"reorganize", fgdbm_reorganize, 0);
-    rb_define_method(cGDBM,"sync", fgdbm_sync, 0);
-    /* rb_define_method(cGDBM,"setopt", fgdbm_setopt, 2); */
-    rb_define_method(cGDBM,"cachesize=", fgdbm_set_cachesize, 1);
-    rb_define_method(cGDBM,"fastmode=", fgdbm_set_fastmode, 1);
-    rb_define_method(cGDBM,"syncmode=", fgdbm_set_syncmode, 1);
+    rb_define_method(rb_cGDBM, "initialize", fgdbm_initialize, -1);
+    rb_define_method(rb_cGDBM, "close", fgdbm_close, 0);
+    rb_define_method(rb_cGDBM, "[]", fgdbm_aref, 1);
+    rb_define_method(rb_cGDBM, "fetch", fgdbm_fetch_m, -1);
+    rb_define_method(rb_cGDBM, "[]=", fgdbm_store, 2);
+    rb_define_method(rb_cGDBM, "store", fgdbm_store, 2);
+    rb_define_method(rb_cGDBM, "index",  fgdbm_index, 1);
+    rb_define_method(rb_cGDBM, "indexes",  fgdbm_indexes, -1);
+    rb_define_method(rb_cGDBM, "indices",  fgdbm_indexes, -1);
+    rb_define_method(rb_cGDBM, "select",  fgdbm_select, -1);
+    rb_define_method(rb_cGDBM, "values_at",  fgdbm_values_at, -1);
+    rb_define_method(rb_cGDBM, "length", fgdbm_length, 0);
+    rb_define_method(rb_cGDBM, "size", fgdbm_length, 0);
+    rb_define_method(rb_cGDBM, "empty?", fgdbm_empty_p, 0);
+    rb_define_method(rb_cGDBM, "each", fgdbm_each_pair, 0);
+    rb_define_method(rb_cGDBM, "each_value", fgdbm_each_value, 0);
+    rb_define_method(rb_cGDBM, "each_key", fgdbm_each_key, 0);
+    rb_define_method(rb_cGDBM, "each_pair", fgdbm_each_pair, 0);
+    rb_define_method(rb_cGDBM, "keys", fgdbm_keys, 0);
+    rb_define_method(rb_cGDBM, "values", fgdbm_values, 0);
+    rb_define_method(rb_cGDBM, "shift", fgdbm_shift, 0);
+    rb_define_method(rb_cGDBM, "delete", fgdbm_delete, 1);
+    rb_define_method(rb_cGDBM, "delete_if", fgdbm_delete_if, 0);
+    rb_define_method(rb_cGDBM, "reject!", fgdbm_delete_if, 0);
+    rb_define_method(rb_cGDBM, "reject", fgdbm_reject, 0);
+    rb_define_method(rb_cGDBM, "clear", fgdbm_clear, 0);
+    rb_define_method(rb_cGDBM,"invert", fgdbm_invert, 0);
+    rb_define_method(rb_cGDBM,"update", fgdbm_update, 1);
+    rb_define_method(rb_cGDBM,"replace", fgdbm_replace, 1);
+    rb_define_method(rb_cGDBM,"reorganize", fgdbm_reorganize, 0);
+    rb_define_method(rb_cGDBM,"sync", fgdbm_sync, 0);
+    /* rb_define_method(rb_cGDBM,"setopt", fgdbm_setopt, 2); */
+    rb_define_method(rb_cGDBM,"cachesize=", fgdbm_set_cachesize, 1);
+    rb_define_method(rb_cGDBM,"fastmode=", fgdbm_set_fastmode, 1);
+    rb_define_method(rb_cGDBM,"syncmode=", fgdbm_set_syncmode, 1);
 
-    rb_define_method(cGDBM, "include?", fgdbm_has_key, 1);
-    rb_define_method(cGDBM, "has_key?", fgdbm_has_key, 1);
-    rb_define_method(cGDBM, "member?", fgdbm_has_key, 1);
-    rb_define_method(cGDBM, "has_value?", fgdbm_has_value, 1);
-    rb_define_method(cGDBM, "key?", fgdbm_has_key, 1);
-    rb_define_method(cGDBM, "value?", fgdbm_has_value, 1);
+    rb_define_method(rb_cGDBM, "include?", fgdbm_has_key, 1);
+    rb_define_method(rb_cGDBM, "has_key?", fgdbm_has_key, 1);
+    rb_define_method(rb_cGDBM, "member?", fgdbm_has_key, 1);
+    rb_define_method(rb_cGDBM, "has_value?", fgdbm_has_value, 1);
+    rb_define_method(rb_cGDBM, "key?", fgdbm_has_key, 1);
+    rb_define_method(rb_cGDBM, "value?", fgdbm_has_value, 1);
 
-    rb_define_method(cGDBM, "to_a", fgdbm_to_a, 0);
-    rb_define_method(cGDBM, "to_hash", fgdbm_to_hash, 0);
+    rb_define_method(rb_cGDBM, "to_a", fgdbm_to_a, 0);
+    rb_define_method(rb_cGDBM, "to_hash", fgdbm_to_hash, 0);
 
     /* flags for gdbm_opn() */
     /*
-    rb_define_const(cGDBM, "READER",  INT2FIX(GDBM_READER));
-    rb_define_const(cGDBM, "WRITER",  INT2FIX(GDBM_WRITER));
-    rb_define_const(cGDBM, "WRCREAT", INT2FIX(GDBM_WRCREAT));
-    rb_define_const(cGDBM, "NEWDB",   INT2FIX(GDBM_NEWDB));
+    rb_define_const(rb_cGDBM, "READER",  INT2FIX(GDBM_READER));
+    rb_define_const(rb_cGDBM, "WRITER",  INT2FIX(GDBM_WRITER));
+    rb_define_const(rb_cGDBM, "WRCREAT", INT2FIX(GDBM_WRCREAT));
+    rb_define_const(rb_cGDBM, "NEWDB",   INT2FIX(GDBM_NEWDB));
     */
-    rb_define_const(cGDBM, "FAST", INT2FIX(GDBM_FAST));
+    rb_define_const(rb_cGDBM, "FAST", INT2FIX(GDBM_FAST));
     /* this flag is obsolete in gdbm 1.8.
        On gdbm 1.8, fast mode is default behavior. */
 
     /* gdbm version 1.8 specific */
 #if defined(GDBM_SYNC)
-    rb_define_const(cGDBM, "SYNC",    INT2FIX(GDBM_SYNC));
+    rb_define_const(rb_cGDBM, "SYNC",    INT2FIX(GDBM_SYNC));
 #endif
 #if defined(GDBM_NOLOCK)
-    rb_define_const(cGDBM, "NOLOCK",  INT2FIX(GDBM_NOLOCK));
+    rb_define_const(rb_cGDBM, "NOLOCK",  INT2FIX(GDBM_NOLOCK));
 #endif
-    rb_define_const(cGDBM, "VERSION",  rb_str_new2(gdbm_version));
+    rb_define_const(rb_cGDBM, "VERSION",  rb_str_new2(gdbm_version));
 }
