@@ -1,5 +1,5 @@
 /*
- * $Id: ossl_x509cert.c,v 1.1 2003/07/23 16:11:29 gotoyuzo Exp $
+ * $Id: ossl_x509cert.c,v 1.3 2003/09/17 09:05:02 gotoyuzo Exp $
  * 'OpenSSL for Ruby' project
  * Copyright (C) 2001-2002  Michal Rokos <m.rokos@sh.cvut.cz>
  * All rights reserved.
@@ -124,7 +124,6 @@ ossl_x509_alloc(VALUE klass)
 
     return obj;
 }
-DEFINE_ALLOC_WRAPPER(ossl_x509_alloc)
 
 static VALUE 
 ossl_x509_initialize(int argc, VALUE *argv, VALUE self)
@@ -137,13 +136,8 @@ ossl_x509_initialize(int argc, VALUE *argv, VALUE self)
 	/* create just empty X509Cert */
 	return self;
     }
+    arg = ossl_to_der_if_possible(arg);
     in = ossl_obj2bio(arg);
-
-    /*
-     * TODO:
-     * Check if we could free old X509
-     X509_free(DATA_PTR(self));
-    */
     x509 = PEM_read_bio_X509(in, (X509 **)&DATA_PTR(self), NULL, NULL);
     if (!x509) {
 	BIO_reset(in);
@@ -179,22 +173,18 @@ static VALUE
 ossl_x509_to_der(VALUE self)
 {
     X509 *x509;
-    BIO *out;
     VALUE str;
-    int status=0;
+    long len;
+    unsigned char *p;
 
     GetX509(self, x509);
-
-    out = BIO_new(BIO_s_mem());
-    if (!out) ossl_raise(eX509CertError, NULL);
-
-    if (!i2d_X509_bio(out, x509)) {
-	BIO_free(out);
+    if ((len = i2d_X509(x509, NULL)) <= 0)
 	ossl_raise(eX509CertError, NULL);
-    }
-    str = ossl_protect_membio2str(out, &status);
-    BIO_free(out);
-    if (status) rb_jump_tag(status);
+    str = rb_str_new(0, len);
+    p = RSTRING(str)->ptr;
+    if (i2d_X509(x509, &p) <= 0)
+	ossl_raise(eX509CertError, NULL);
+    ossl_str_adjust(str, p);
     
     return str;
 }
@@ -205,7 +195,6 @@ ossl_x509_to_pem(VALUE self)
     X509 *x509;
     BIO *out;
     VALUE str;
-    int status=0;
 	
     GetX509(self, x509);
     out = BIO_new(BIO_s_mem());
@@ -215,9 +204,7 @@ ossl_x509_to_pem(VALUE self)
 	BIO_free(out);
 	ossl_raise(eX509CertError, NULL);
     }
-    str = ossl_protect_membio2str(out, &status);
-    BIO_free(out);
-    if (status) rb_jump_tag(status);
+    str = ossl_membio2str(out);
 
     return str;
 }
@@ -228,7 +215,6 @@ ossl_x509_to_text(VALUE self)
     X509 *x509;
     BIO *out;
     VALUE str;
-    int status=0;
 	
     GetX509(self, x509);
 
@@ -239,9 +225,7 @@ ossl_x509_to_text(VALUE self)
 	BIO_free(out);
 	ossl_raise(eX509CertError, NULL);
     }
-    str = ossl_protect_membio2str(out, &status);
-    BIO_free(out);
-    if (status) rb_jump_tag(status);
+    str = ossl_membio2str(out);
 
     return str;
 }
@@ -324,7 +308,6 @@ ossl_x509_get_signature_algorithm(VALUE self)
     X509 *x509;
     BIO *out;
     VALUE str;
-    int status=0;
 
     GetX509(self, x509);
 	
@@ -335,9 +318,7 @@ ossl_x509_get_signature_algorithm(VALUE self)
 	BIO_free(out);
 	ossl_raise(eX509CertError, NULL);
     }
-    str = ossl_protect_membio2str(out, &status);
-    BIO_free(out);
-    if (status) rb_jump_tag(status);
+    str = ossl_membio2str(out);
 
     return str;
 }
