@@ -1,5 +1,5 @@
 /*
- * $Id: ossl_x509ext.c,v 1.1 2003/07/23 16:11:29 gotoyuzo Exp $
+ * $Id: ossl_x509ext.c,v 1.8 2003/11/15 13:19:17 gotoyuzo Exp $
  * 'OpenSSL for Ruby' project
  * Copyright (C) 2001-2002  Michal Rokos <m.rokos@sh.cvut.cz>
  * All rights reserved.
@@ -97,13 +97,7 @@ DupX509ExtPtr(VALUE obj)
 static void
 ossl_x509extfactory_free(X509V3_CTX *ctx)
 {
-    if (ctx) {
-	if (ctx->issuer_cert)	X509_free(ctx->issuer_cert);
-	if (ctx->subject_cert)	X509_free(ctx->subject_cert);
-	if (ctx->crl)		X509_CRL_free(ctx->crl);
-	if (ctx->subject_req)	X509_REQ_free(ctx->subject_req);
-	OPENSSL_free(ctx);
-    }
+    OPENSSL_free(ctx);
 }
 
 static VALUE 
@@ -116,7 +110,6 @@ ossl_x509extfactory_alloc(VALUE klass)
 
     return obj;
 }
-DEFINE_ALLOC_WRAPPER(ossl_x509extfactory_alloc)
 
 static VALUE 
 ossl_x509extfactory_set_issuer_cert(VALUE self, VALUE cert)
@@ -124,7 +117,8 @@ ossl_x509extfactory_set_issuer_cert(VALUE self, VALUE cert)
     X509V3_CTX *ctx;
 
     GetX509ExtFactory(self, ctx);
-    ctx->issuer_cert = DupX509CertPtr(cert); /* DUP NEEDED */
+    rb_iv_set(self, "@issuer_certificate", cert);
+    ctx->issuer_cert = GetX509CertPtr(cert); /* NO DUP NEEDED */
 
     return cert;
 }
@@ -135,7 +129,8 @@ ossl_x509extfactory_set_subject_cert(VALUE self, VALUE cert)
     X509V3_CTX *ctx;
 
     GetX509ExtFactory(self, ctx);
-    ctx->subject_cert = DupX509CertPtr(cert); /* DUP NEEDED */
+    rb_iv_set(self, "@subject_certificate", cert);
+    ctx->subject_cert = GetX509CertPtr(cert); /* NO DUP NEEDED */
 
     return cert;
 }
@@ -146,7 +141,8 @@ ossl_x509extfactory_set_subject_req(VALUE self, VALUE req)
     X509V3_CTX *ctx;
 
     GetX509ExtFactory(self, ctx);
-    ctx->subject_req = DupX509ReqPtr(req);
+    rb_iv_set(self, "@subject_request", req);
+    ctx->subject_req = GetX509ReqPtr(req); /* NO DUP NEEDED */
 
     return req;
 }
@@ -157,11 +153,30 @@ ossl_x509extfactory_set_crl(VALUE self, VALUE crl)
     X509V3_CTX *ctx;
 
     GetX509ExtFactory(self, ctx);
-    ctx->crl = DupX509CRLPtr(crl);
+    rb_iv_set(self, "@crl", crl);
+    ctx->crl = GetX509CRLPtr(crl); /* NO DUP NEEDED */
 
     return crl;
 }
 
+static VALUE
+ossl_x509extfactory_set_config(VALUE self, VALUE config)
+{
+#ifdef HAVE_X509V3_SET_NCONF
+    X509V3_CTX *ctx;
+    CONF *conf;
+
+    GetX509ExtFactory(self, ctx);
+    rb_iv_set(self, "@config", config);
+    conf = GetConfigPtr(config);  /* NO DUP NEEDED */
+    X509V3_set_nconf(ctx, conf);
+
+    return config;
+#else
+    rb_notimplement();
+#endif
+}
+
 static VALUE 
 ossl_x509extfactory_initialize(int argc, VALUE *argv, VALUE self)
 {
@@ -170,20 +185,17 @@ ossl_x509extfactory_initialize(int argc, VALUE *argv, VALUE self)
 	
     /*GetX509ExtFactory(self, ctx);*/
 
-    rb_scan_args(argc, argv, "04", &issuer_cert, &subject_cert, &subject_req, &crl);
-
-    if (!NIL_P(issuer_cert)) {
+    rb_scan_args(argc, argv, "04",
+		 &issuer_cert, &subject_cert, &subject_req, &crl);
+    if (!NIL_P(issuer_cert))
 	ossl_x509extfactory_set_issuer_cert(self, issuer_cert);
-    }
-    if (!NIL_P(subject_cert)) {
+    if (!NIL_P(subject_cert))
 	ossl_x509extfactory_set_subject_cert(self, subject_cert);
-    }
-    if (!NIL_P(subject_req)) {
+    if (!NIL_P(subject_req))
 	ossl_x509extfactory_set_subject_req(self, subject_req);
-    }
-    if (!NIL_P(crl)) {
+    if (!NIL_P(crl))
 	ossl_x509extfactory_set_crl(self, crl);
-    }
+
     return self;
 }
 
@@ -196,46 +208,29 @@ ossl_x509extfactory_initialize(int argc, VALUE *argv, VALUE self)
  * ["ln", "value"] => not critical
  */
 static VALUE 
-ossl_x509extfactory_create_ext_from_array(VALUE self, VALUE ary)
+ossl_x509extfactory_create_ext(int argc, VALUE *argv, VALUE self)
 {
     X509V3_CTX *ctx;
     X509_EXTENSION *ext;
+    VALUE oid, value, critical, valstr, obj;
     int nid;
-    char *value;
-    VALUE item, obj;
 
     GetX509ExtFactory(self, ctx);
-    Check_Type(ary, T_ARRAY);
-    if ((RARRAY(ary)->len) < 2 || (RARRAY(ary)->len > 3)) { /*2 or 3 allowed*/
-	ossl_raise(eX509ExtError, "unsupported structure");
+    rb_scan_args(argc, argv, "21", &oid, &value, &critical);
+    StringValue(oid);
+    StringValue(value);
+    if(NIL_P(critical)) critical = Qfalse;
+
+    nid = OBJ_ln2nid(RSTRING(oid)->ptr);
+    if(!nid) nid = OBJ_sn2nid(RSTRING(oid)->ptr);
+    if(!nid) ossl_raise(eX509ExtError, "unknown OID `%s'", RSTRING(oid)->ptr);
+    valstr = rb_str_new2(RTEST(critical) ? "critical," : "");
+    rb_str_append(valstr, value);
+    ext = X509V3_EXT_conf_nid(NULL, ctx, nid, RSTRING(valstr)->ptr);
+    if (!ext){
+	ossl_raise(eX509ExtError, "%s = %s",
+		   RSTRING(oid)->ptr, RSTRING(value)->ptr);
     }
-    /* key [0] */
-    item = RARRAY(ary)->ptr[0];
-    StringValue(item);
-    if (!(nid = OBJ_ln2nid(RSTRING(item)->ptr))) {
-	if (!(nid = OBJ_sn2nid(RSTRING(item)->ptr))) {
-	    ossl_raise(eX509ExtError, NULL);
-	}
-    }
-    /* data [1] */
-    item = RARRAY(ary)->ptr[1];
-    StringValue(item);
-    /* (optional) critical [2] */
-    if (RARRAY(ary)->len == 3 && RARRAY(ary)->ptr[2] == Qtrue) {
-	if (!(value = OPENSSL_malloc(strlen("critical,") +
-				     (RSTRING(item)->len) + 1))) {
-	    ossl_raise(eX509ExtError, "malloc error");
-	}
-	strcpy(value, "critical,");
-	strncat(value, RSTRING(item)->ptr, RSTRING(item)->len);
-    } else {
-	value = strdup(StringValuePtr(item));
-    }
-    if (!(ext = X509V3_EXT_conf_nid(NULL, ctx, nid, value))) {
-	OPENSSL_free(value);
-	ossl_raise(eX509ExtError, NULL);
-    }
-    OPENSSL_free(value);
     WrapX509Ext(cX509Ext, obj, ext);
 
     return obj;
@@ -244,6 +239,99 @@ ossl_x509extfactory_create_ext_from_array(VALUE self, VALUE ary)
 /*
  * Ext
  */
+static VALUE
+ossl_x509ext_alloc(VALUE klass)
+{
+    X509_EXTENSION *ext;
+    VALUE obj;
+
+    if(!(ext = X509_EXTENSION_new())){
+	ossl_raise(eX509ExtError, NULL);
+    }
+    WrapX509Ext(klass, obj, ext);
+
+    return obj;
+}
+
+static VALUE
+ossl_x509ext_initialize(int argc, VALUE *argv, VALUE self)
+{
+    VALUE oid, value, critical;
+    unsigned char *p;
+    X509_EXTENSION *ext;
+
+    if(rb_scan_args(argc, argv, "12", &oid, &value, &critical) == 1){
+	/* evaluate oid as a DER string */
+	oid = ossl_to_der_if_possible(oid);
+	StringValue(oid);
+	GetX509Ext(self, ext);
+	p  = RSTRING(oid)->ptr;
+	if(!d2i_X509_EXTENSION(&ext, &p, RSTRING(oid)->len))
+	    ossl_raise(eX509ExtError, NULL);
+	return self;
+    }
+    rb_funcall(self, rb_intern("oid="), 1, oid);
+    rb_funcall(self, rb_intern("value="), 1, value);
+    if(argc > 2) rb_funcall(self, rb_intern("critical="), 1, critical);
+
+    return self;
+}
+
+static VALUE
+ossl_x509ext_set_oid(VALUE self, VALUE oid)
+{
+    X509_EXTENSION *ext;
+    ASN1_OBJECT *obj;
+    char *s;
+
+    GetX509Ext(self, ext);
+    s = StringValuePtr(oid);
+    obj = OBJ_txt2obj(s, 0);
+    if(!obj) obj = OBJ_txt2obj(s, 1);
+    if(!obj) ossl_raise(eX509ExtError, NULL);
+    X509_EXTENSION_set_object(ext, obj);
+
+    return oid;
+}
+
+static VALUE
+ossl_x509ext_set_value(VALUE self, VALUE data)
+{
+    X509_EXTENSION *ext;
+    ASN1_OCTET_STRING *asn1s;
+    char *s;
+
+    GetX509Ext(self, ext);
+    data = ossl_to_der_if_possible(data);
+    StringValue(data);
+    if(!(s = OPENSSL_malloc(RSTRING(data)->len)))
+	ossl_raise(eX509ExtError, "malloc error");
+    memcpy(s, RSTRING(data)->ptr, RSTRING(data)->len);
+    if(!(asn1s = ASN1_OCTET_STRING_new())){
+	free(s);
+	ossl_raise(eX509ExtError, NULL);
+    }
+    if(!M_ASN1_OCTET_STRING_set(asn1s, s, RSTRING(data)->len)){
+	free(s);
+	ASN1_OCTET_STRING_free(asn1s);
+	ossl_raise(eX509ExtError, NULL);
+    }
+    X509_EXTENSION_set_data(ext, asn1s);
+
+    return data;
+}
+
+static VALUE
+ossl_x509ext_set_critical(VALUE self, VALUE flag)
+{
+    X509_EXTENSION *ext;
+
+    GetX509Ext(self, ext);
+    X509_EXTENSION_set_critical(ext, RTEST(flag) ? 1 : 0);
+
+    return flag;
+}
+
 static VALUE 
 ossl_x509ext_get_oid(VALUE obj)
 {
@@ -251,7 +339,7 @@ ossl_x509ext_get_oid(VALUE obj)
     ASN1_OBJECT *extobj;
     BIO *out;
     VALUE ret;
-    int nid, status = 0;
+    int nid;
 
     GetX509Ext(obj, ext);
     extobj = X509_EXTENSION_get_object(ext);
@@ -261,9 +349,7 @@ ossl_x509ext_get_oid(VALUE obj)
 	if (!(out = BIO_new(BIO_s_mem())))
 	    ossl_raise(eX509ExtError, NULL);
 	i2a_ASN1_OBJECT(out, extobj);
-	ret = ossl_protect_membio2str(out, &status);
-	BIO_free(out);
-	if(status) rb_jump_tag(status);
+	ret = ossl_membio2str(out);
     }
 
     return ret;
@@ -275,16 +361,13 @@ ossl_x509ext_get_value(VALUE obj)
     X509_EXTENSION *ext;
     BIO *out;
     VALUE ret;
-    int status = 0;
 
     GetX509Ext(obj, ext);
     if (!(out = BIO_new(BIO_s_mem())))
 	ossl_raise(eX509ExtError, NULL);
     if (!X509V3_EXT_print(out, ext, 0, 0))
 	M_ASN1_OCTET_STRING_print(out, ext->value);
-    ret = ossl_protect_membio2str(out, &status);
-    BIO_free(out);
-    if(status) rb_jump_tag(status);
+    ret = ossl_membio2str(out);
 
     return ret;
 }
@@ -302,15 +385,17 @@ ossl_x509ext_to_der(VALUE obj)
 {
     X509_EXTENSION *ext;
     unsigned char *p;
-    int len;
+    long len;
     VALUE str;
 
     GetX509Ext(obj, ext);
-    p = NULL;
-    if((len = i2d_X509_EXTENSION(ext, &p)) < 0)
-        ossl_raise(eX509ExtError, NULL);
-    str = rb_str_new(p, len);
-    free(p);
+    if((len = i2d_X509_EXTENSION(ext, NULL)) <= 0)
+	ossl_raise(eX509ExtError, NULL);
+    str = rb_str_new(0, len);
+    p = RSTRING(str)->ptr;
+    if(i2d_X509_EXTENSION(ext, &p) < 0)
+	ossl_raise(eX509ExtError, NULL);
+    ossl_str_adjust(str, p);
 
     return str;
 }
@@ -328,16 +413,25 @@ Init_ossl_x509ext()
     rb_define_alloc_func(cX509ExtFactory, ossl_x509extfactory_alloc);
     rb_define_method(cX509ExtFactory, "initialize", ossl_x509extfactory_initialize, -1);
 	
+    rb_attr(cX509ExtFactory, rb_intern("issuer_certificate"), 1, 0, Qfalse);
+    rb_attr(cX509ExtFactory, rb_intern("subject_certificate"), 1, 0, Qfalse);
+    rb_attr(cX509ExtFactory, rb_intern("subject_request"), 1, 0, Qfalse);
+    rb_attr(cX509ExtFactory, rb_intern("crl"), 1, 0, Qfalse);
+    rb_attr(cX509ExtFactory, rb_intern("config"), 1, 0, Qfalse);
+
     rb_define_method(cX509ExtFactory, "issuer_certificate=", ossl_x509extfactory_set_issuer_cert, 1);
     rb_define_method(cX509ExtFactory, "subject_certificate=", ossl_x509extfactory_set_subject_cert, 1);
     rb_define_method(cX509ExtFactory, "subject_request=", ossl_x509extfactory_set_subject_req, 1);
     rb_define_method(cX509ExtFactory, "crl=", ossl_x509extfactory_set_crl, 1);
-    rb_define_method(cX509ExtFactory, "create_ext_from_array", ossl_x509extfactory_create_ext_from_array, 1);
+    rb_define_method(cX509ExtFactory, "config=", ossl_x509extfactory_set_config, 1);
+    rb_define_method(cX509ExtFactory, "create_ext", ossl_x509extfactory_create_ext, -1);
 	
     cX509Ext = rb_define_class_under(mX509, "Extension", rb_cObject);
-    rb_undef_method(CLASS_OF(cX509Ext), "new");
-/*  rb_define_alloc_func(cX509Ext, ossl_x509ext_alloc); */
-/*  rb_define_method(cX509Ext, "initialize", ossl_x509ext_initialize, -1); */
+    rb_define_alloc_func(cX509Ext, ossl_x509ext_alloc);
+    rb_define_method(cX509Ext, "initialize", ossl_x509ext_initialize, -1);
+    rb_define_method(cX509Ext, "oid=", ossl_x509ext_set_oid, 1);
+    rb_define_method(cX509Ext, "value=", ossl_x509ext_set_value, 1);
+    rb_define_method(cX509Ext, "critical=", ossl_x509ext_set_critical, 1);
     rb_define_method(cX509Ext, "oid", ossl_x509ext_get_oid, 0);
     rb_define_method(cX509Ext, "value", ossl_x509ext_get_value, 0);
     rb_define_method(cX509Ext, "critical?", ossl_x509ext_get_critical, 0);
