@@ -2,8 +2,8 @@
 
   socket.c -
 
-  $Author: nobu $
-  $Date: 2003/08/01 07:02:54 $
+  $Author: eban $
+  $Date: 2003/12/14 10:04:34 $
   created at: Thu Mar 31 12:21:29 JST 1994
 
   Copyright (C) 1993-2001 Yukihiro Matsumoto
@@ -13,6 +13,7 @@
 #include "ruby.h"
 #include "rubyio.h"
 #include "rubysig.h"
+#include "util.h"
 #include <stdio.h>
 #include <sys/types.h>
 
@@ -31,6 +32,9 @@
 # include <sys/socket.h>
 #endif
 #include <netinet/in.h>
+#ifdef HAVE_NETINET_IN_SYSTM_H
+# include <netinet/in_systm.h>
+#endif
 #ifdef HAVE_NETINET_TCP_H
 # include <netinet/tcp.h>
 #endif
@@ -492,7 +496,7 @@ bsock_do_not_rev_lookup_set(self, val)
 }
 
 static void
-mkipaddr0(addr, buf, len)
+make_ipaddr0(addr, buf, len)
     struct sockaddr *addr;
     char *buf;
     size_t len;
@@ -506,17 +510,17 @@ mkipaddr0(addr, buf, len)
 }
 
 static VALUE
-mkipaddr(addr)
+make_ipaddr(addr)
     struct sockaddr *addr;
 {
     char buf[1024];
 
-    mkipaddr0(addr, buf, sizeof(buf));
+    make_ipaddr0(addr, buf, sizeof(buf));
     return rb_str_new2(buf);
 }
 
 static void
-mkinetaddr(host, buf, len)
+make_inetaddr(host, buf, len)
     long host;
     char *buf;
     size_t len;
@@ -527,7 +531,7 @@ mkinetaddr(host, buf, len)
     sin.sin_family = AF_INET;
     SET_SIN_LEN(&sin, sizeof(sin));
     sin.sin_addr.s_addr = host;
-    mkipaddr0((struct sockaddr*)&sin, buf, len);
+    make_ipaddr0((struct sockaddr*)&sin, buf, len);
 }
 
 static int
@@ -546,6 +550,75 @@ str_isnumber(p)
        return 0;
 }
 
+static char *
+host_str(host, hbuf, len)
+    VALUE host;
+    char *hbuf;
+    size_t len;
+{
+    if (NIL_P(host)) {
+	return NULL;
+    }
+    else if (rb_obj_is_kind_of(host, rb_cInteger)) {
+	long i = NUM2LONG(host);
+
+	make_inetaddr(htonl(i), hbuf, len);
+	return hbuf;
+    }
+    else {
+	char *name;
+
+	SafeStringValue(host);
+	name = RSTRING(host)->ptr;
+	if (!name || *name == 0 || (name[0] == '<' && strcmp(name, "<any>") == 0)) {
+	    make_inetaddr(INADDR_ANY, hbuf, len);
+	}
+	else if (name[0] == '<' && strcmp(name, "<broadcast>") == 0) {
+	    make_inetaddr(INADDR_BROADCAST, hbuf, len);
+	}
+	else if (strlen(name) >= len) {
+	    rb_raise(rb_eArgError, "hostname too long (%d)", strlen(name));
+	}
+	else {
+	    strcpy(hbuf, name);
+	}
+	return hbuf;
+    }
+}
+
+static char *
+port_str(port, pbuf, len)
+    VALUE port;
+    char *pbuf;
+    size_t len;
+{
+    if (NIL_P(port)) {
+	return 0;
+    }
+    else if (FIXNUM_P(port)) {
+	snprintf(pbuf, len, "%ld", FIX2LONG(port));
+	return pbuf;
+    }
+    else {
+	char *serv;
+
+	SafeStringValue(port);
+	serv = RSTRING(port)->ptr;
+	if (strlen(serv) >= len) {
+	    rb_raise(rb_eArgError, "service name too long (%d)", strlen(serv));
+	}
+	strcpy(pbuf, serv);
+	return pbuf;
+    }
+}
+
+#ifndef NI_MAXHOST
+# define 1025
+#endif
+#ifndef NI_MAXSERV
+# define 32
+#endif
+
 static struct addrinfo*
 sock_addrinfo(host, port, socktype, flags)
     VALUE host, port;
@@ -554,47 +627,10 @@ sock_addrinfo(host, port, socktype, flags)
     struct addrinfo hints, *hintsp, *res;
     char *hostp, *portp;
     int error;
-    char hbuf[1024], pbuf[32];
+    char hbuf[NI_MAXHOST], pbuf[NI_MAXSERV];
 
-    if (NIL_P(host)) {
-	hostp = NULL;
-    }
-    else if (rb_obj_is_kind_of(host, rb_cInteger)) {
-	long i = NUM2LONG(host);
-
-	mkinetaddr(htonl(i), hbuf, sizeof(hbuf));
-	hostp = hbuf;
-    }
-    else {
-	char *name;
-
-	SafeStringValue(host);
-	name = RSTRING(host)->ptr;
-	if (!name || *name == 0 || (name[0] == '<' && strcmp(name, "<any>") == 0)) {
-	    mkinetaddr(INADDR_ANY, hbuf, sizeof(hbuf));
-	}
-	else if (name[0] == '<' && strcmp(name, "<broadcast>") == 0) {
-	    mkinetaddr(INADDR_BROADCAST, hbuf, sizeof(hbuf));
-	}
-	else if (strlen(name) >= sizeof(hbuf)) {
-	    rb_raise(rb_eArgError, "hostname too long (%d)", strlen(name));
-	}
-	else {
-	    strcpy(hbuf, name);
-	}
-	hostp = hbuf;
-    }
-    if (NIL_P(port)) {
-	portp = 0;
-    }
-    else if (FIXNUM_P(port)) {
-	snprintf(pbuf, sizeof(pbuf), "%d", FIX2INT(port));
-	portp = pbuf;
-    }
-    else {
-	SafeStringValue(port);
-	portp = RSTRING(port)->ptr;
-    }
+    hostp = host_str(host, hbuf, sizeof(hbuf));
+    portp = port_str(port, pbuf, sizeof(pbuf));
 
     if (socktype == 0 && flags == 0 && str_isnumber(portp)) {
        socktype = SOCK_DGRAM;
@@ -617,18 +653,6 @@ sock_addrinfo(host, port, socktype, flags)
     return res;
 }
 
-static void
-setipaddr(name, addr)
-    VALUE name;
-    struct sockaddr_storage *addr;
-{
-    struct addrinfo *res = sock_addrinfo(name, Qnil, SOCK_STREAM, 0);
-
-    /* just take the first one */
-    memcpy(addr, res->ai_addr, res->ai_addrlen);
-    freeaddrinfo(res);
-}
-
 static VALUE
 ipaddr(sockaddr)
     struct sockaddr *sockaddr;
@@ -703,17 +727,6 @@ ruby_socket(domain, type, proto)
     return fd;
 }
 
-static void
-thread_read_select(fd)
-    int fd;
-{
-    fd_set fds;
-
-    FD_ZERO(&fds);
-    FD_SET(fd, &fds);
-    rb_thread_select(fd+1, &fds, 0, 0, 0);
-}
-
 static void
 thread_write_select(fd)
     int fd;
@@ -731,6 +744,14 @@ thread_write_select(fd)
 #ifdef __APPLE__
 #define WAIT_IN_PROGRESS 10
 #endif
+#ifdef __linux__
+/* returns correct error */
+#define WAIT_IN_PROGRESS 0
+#endif
+#ifndef WAIT_IN_PROGRESS
+/* BSD origin code apparently has a problem */
+#define WAIT_IN_PROGRESS 1
+#endif
 
 static int
 ruby_connect(fd, sockaddr, len, socks)
@@ -741,8 +762,9 @@ ruby_connect(fd, sockaddr, len, socks)
 {
     int status;
     int mode;
-#ifdef WAIT_IN_PROGRESS
+#if WAIT_IN_PROGRESS > 0
     int wait_in_progress = -1;
+    int sockerr, sockerrlen;
 #endif
 
 #if defined(HAVE_FCNTL)
@@ -779,26 +801,35 @@ ruby_connect(fd, sockaddr, len, socks)
 #ifdef EINPROGRESS
 	      case EINPROGRESS:
 #endif
+#if WAIT_IN_PROGRESS > 0
+		sockerrlen = sizeof(sockerr);
+		status = getsockopt(fd, SOL_SOCKET, SO_ERROR, (void *)&sockerr, &sockerrlen);
+		if (status) break;
+		if (sockerr) {
+		    status = -1;
+		    errno = sockerr;
+		    break;
+		}
+#endif
 #ifdef EALREADY
 	      case EALREADY:
 #endif
-#ifdef WAIT_IN_PROGRESS
+#if WAIT_IN_PROGRESS > 0
 		wait_in_progress = WAIT_IN_PROGRESS;
 #endif
 		thread_write_select(fd);
 		continue;
 
-#ifdef WAIT_IN_PROGRESS
+#if WAIT_IN_PROGRESS > 0
 	      case EINVAL:
 		if (wait_in_progress-- > 0) {
-		    int sockerr, sockerrlen = sizeof(sockerr);
-
 		    /*
 		     * connect() after EINPROGRESS returns EINVAL on
 		     * some platforms, need to check true error
 		     * status.
 		     */
-		    status = getsockopt(fd, SOL_SOCKET, SO_ERROR, &sockerr, &sockerrlen);
+		    sockerrlen = sizeof(sockerr);
+		    status = getsockopt(fd, SOL_SOCKET, SO_ERROR, (void *)&sockerr, &sockerrlen);
 		    if (!status && !sockerr) {
 			struct timeval tv = {0, 100000};
 			rb_thread_wait_for(tv);
@@ -827,46 +858,6 @@ ruby_connect(fd, sockaddr, len, socks)
     }
 }
 
-static void
-load_addr_info(h, serv, type, res)
-    VALUE h, serv;
-    int type;
-    struct addrinfo **res;
-{
-    char *host;
-    char pbuf[1024], *portp;
-    struct addrinfo hints;
-    int error;
-
-    if (!NIL_P(h)) {
-	SafeStringValue(h);
-	host = RSTRING(h)->ptr;
-    }
-    else {
-	host = NULL;
-    }
-    if (FIXNUM_P(serv)) {
-	snprintf(pbuf, sizeof(pbuf), "%u", FIX2UINT(serv));
-	portp = pbuf;
-    }
-    else {
-	SafeStringValue(serv);
-	if (RSTRING(serv)->len >= sizeof(pbuf))
-	    rb_raise(rb_eArgError, "servicename too long (%ld)", RSTRING(serv)->len);
-	strcpy(pbuf, RSTRING(serv)->ptr);
-	portp = pbuf;
-    }
-    MEMZERO(&hints, struct addrinfo, 1);
-    hints.ai_family = PF_UNSPEC;
-    if (type == INET_SERVER) {
-	hints.ai_flags = AI_PASSIVE;
-    }
-    error = getaddrinfo(host, portp, &hints, res);
-    if (error) {
-	rb_raise(rb_eSocket, "getaddrinfo: %s", gai_strerror(error));
-    }
-}
-
 struct inetsock_arg
 {
     VALUE sock;
@@ -902,7 +893,7 @@ init_inetsock_internal(arg)
 {
     int type = arg->type;
     struct addrinfo *res;
-    int fd, status;
+    int fd, status = 0;
     char *syscall;
 
     arg->remote.res = sock_addrinfo(arg->remote.host, arg->remote.serv, SOCK_STREAM,
@@ -1034,110 +1025,66 @@ socks_s_close(sock)
 #endif
 #endif
 
-/*
- * NOTE: using gethostbyname() against AF_INET6 is a bad idea, as it
- * does not initialize sin_flowinfo nor sin_scope_id properly.
- */
-
-struct hostent*
-sock_hostbyname(host)
-    VALUE host;
+static VALUE
+make_hostent(addr, ipaddr)
+    struct addrinfo *addr;
+    VALUE (*ipaddr) _((struct sockaddr*, size_t));
 {
-    struct sockaddr_storage addr;
+    struct addrinfo *ai;
     struct hostent *h;
+    VALUE ary, names;
+    char **pch;
 
-    rb_secure(3);
-    setipaddr(host, &addr);
-    switch (addr.ss_family) {
-      case AF_INET:
-      {
-	struct sockaddr_in *sin;
-	sin = (struct sockaddr_in*)&addr;
-	h = gethostbyaddr((char*)&sin->sin_addr,
-			  sizeof(sin->sin_addr),
-			  sin->sin_family);
-	break;
-      }
-#ifdef INET6
-      case AF_INET6:
-      {
-	struct sockaddr_in6 *sin6;
-	sin6 = (struct sockaddr_in6*)&addr;
-	h = gethostbyaddr((char*)&sin6->sin6_addr,
-			  sizeof(sin6->sin6_addr),
-			  sin6->sin6_family);
-	break;
-      }
-#endif
-      default:
-	h = NULL;
-	break;
+    ary = rb_ary_new();
+    rb_ary_push(ary, rb_str_new2(addr->ai_canonname));
+#if defined(HAVE_GETIPNODEBYNAME)
+    {
+	int error;
+
+	h = getipnodebyname(addr->ai_canonname, addr->ai_family, AI_ALL, &error);
     }
-
-    if (h == NULL) {
-#ifdef HAVE_HSTERROR
-	extern int h_errno;
-	rb_raise(rb_eSocket, "%s", (char*)hsterror(h_errno));
+#elif defined(HAVE_GETHOSTBYNAME2)
+    h = gethostbyname2(addr->ai_canonname, addr->ai_family);
 #else
-	rb_raise(rb_eSocket, "host not found");
+    h = gethostbyname(addr->ai_canonname);
+#endif
+    if (h) {
+	names = rb_ary_new();
+	if (h->h_aliases != NULL) {
+	    for (pch = h->h_aliases; *pch; pch++) {
+		rb_ary_push(names, rb_str_new2(*pch));
+	    }
+	}
+#if defined(HAVE_GETIPNODEBYNAME)
+	freehostent(h);
 #endif
     }
-    return h;
+    else {
+	names = rb_ary_new2(0);
+    }
+    rb_ary_push(ary, names);
+    rb_ary_push(ary, INT2NUM(addr->ai_family));
+    for (ai = addr; ai; ai = ai->ai_next) {
+	rb_ary_push(ary, (*ipaddr)(ai->ai_addr, ai->ai_addrlen));
+    }
+
+    return ary;
+}
+
+VALUE
+tcp_sockaddr(addr, len)
+    struct sockaddr *addr;
+    size_t len;
+{
+    return make_ipaddr(addr);
 }
 
 static VALUE
 tcp_s_gethostbyname(obj, host)
     VALUE obj, host;
 {
-    struct hostent *h = sock_hostbyname(host);
-    VALUE ary, names;
-    char **pch;
-
-    ary = rb_ary_new();
-    rb_ary_push(ary, rb_str_new2(h->h_name));
-    names = rb_ary_new();
-    rb_ary_push(ary, names);
-    for (pch = h->h_aliases; *pch; pch++) {
-	rb_ary_push(names, rb_str_new2(*pch));
-    }
-    rb_ary_push(ary, INT2NUM(h->h_addrtype));
-#ifdef h_addr
-    for (pch = h->h_addr_list; *pch; pch++) {
-	switch (h->h_length) {
-	  case 4: /* AF_INET */ {
-	    struct sockaddr_in sin;
-
-	    MEMZERO(&sin, struct sockaddr_in, 1);
-	    sin.sin_family = AF_INET;
-	    SET_SIN_LEN(&sin, sizeof(sin));
-	    memcpy((char*)&sin.sin_addr, *pch, h->h_length);
-	    rb_ary_push(ary, mkipaddr((struct sockaddr*)&sin));
-	    break;
-	  }
-#ifdef INET6
-	  case 8: /* AF_INET6 */ {
-	    struct sockaddr_in6 sin6;
-
-	    MEMZERO(&sin6, struct sockaddr_in6, 1);
-	    sin6.sin6_family = AF_INET6;
-#ifdef SIN6_LEN
-	    sin6.sin6_len = sizeof(sin6);
-#endif
-	    memcpy((char*)&sin6.sin6_addr, *pch, h->h_length);
-	    rb_ary_push(ary, mkipaddr((struct sockaddr*)&sin6));
-	    break;
-	  }
-#endif
-	  default:
-	    break;
-	}
-    }
-#else
-    memcpy((char*)&addr.sin_addr, h->h_addr, h->h_length);
-    rb_ary_push(ary, mkipaddr((struct sockaddr*)&addr));
-#endif
-
-    return ary;
+    rb_secure(3);
+    return make_hostent(sock_addrinfo(host, Qnil, SOCK_STREAM, AI_CANONNAME), tcp_sockaddr);
 }
 
 static VALUE
@@ -1329,9 +1276,13 @@ ip_s_getaddress(obj, host)
     VALUE obj, host;
 {
     struct sockaddr_storage addr;
+    struct addrinfo *res = sock_addrinfo(host, Qnil, SOCK_STREAM, 0);
 
-    setipaddr(host, &addr);
-    return mkipaddr((struct sockaddr*)&addr);
+    /* just take the first one */
+    memcpy(&addr, res->ai_addr, res->ai_addrlen);
+    freeaddrinfo(res);
+
+    return make_ipaddr((struct sockaddr*)&addr);
 }
 
 static VALUE
@@ -1481,13 +1432,6 @@ unix_path(sock)
     return rb_str_new2(fptr->path);
 }
 
-static VALUE
-unix_svr_s_open(klass, path)
-    VALUE klass, path;
-{
-    return init_unixsock(rb_obj_alloc(klass), path, 1);
-}
-
 static VALUE
 unix_svr_init(sock, path)
     VALUE sock, path;
@@ -1569,6 +1513,19 @@ unix_send_io(sock, val)
 #endif
 }
 
+#if defined(HAVE_RECVMSG) && (defined(HAVE_ST_MSG_CONTROL) || defined(HAVE_ST_MSG_ACCRIGHTS))
+static void
+thread_read_select(fd)
+    int fd;
+{
+    fd_set fds;
+
+    FD_ZERO(&fds);
+    FD_SET(fd, &fds);
+    rb_thread_select(fd+1, &fds, 0, 0, 0);
+}
+#endif
+
 static VALUE
 unix_recv_io(argc, argv, sock)
     int argc;
@@ -2005,41 +1962,7 @@ sock_gethostname(obj)
 #endif
 
 static VALUE
-sock_mkhostent(h)
-    struct hostent *h;
-{
-    char **pch;
-    VALUE ary, names;
-
-    if (h == NULL) {
-#ifdef HAVE_HSTRERROR
-	extern int h_errno;
-	rb_raise(rb_eSocket, "%s", (char*)hstrerror(h_errno));
-#else
-	rb_raise(rb_eSocket, "host not found");
-#endif
-    }
-    ary = rb_ary_new();
-    rb_ary_push(ary, rb_str_new2(h->h_name));
-    names = rb_ary_new();
-    rb_ary_push(ary, names);
-    for (pch = h->h_aliases; *pch; pch++) {
-	rb_ary_push(names, rb_str_new2(*pch));
-    }
-    rb_ary_push(ary, INT2NUM(h->h_addrtype));
-#ifdef h_addr
-    for (pch = h->h_addr_list; *pch; pch++) {
-	rb_ary_push(ary, rb_str_new(*pch, h->h_length));
-    }
-#else
-    rb_ary_push(ary, rb_str_new(h->h_addr, h->h_length));
-#endif
-
-    return ary;
-}
-
-static VALUE
-mkaddrinfo(res0)
+make_addrinfo(res0)
     struct addrinfo *res0;
 {
     VALUE base, ary;
@@ -2059,11 +1982,20 @@ mkaddrinfo(res0)
     return base;
 }
 
+VALUE
+sock_sockaddr(addr, len)
+    struct sockaddr *addr;
+    size_t len;
+{
+    return rb_str_new((char*)addr, len);
+}
+
 static VALUE
 sock_s_gethostbyname(obj, host)
     VALUE obj, host;
 {
-    return sock_mkhostent(sock_hostbyname(host));
+    rb_secure(3);
+    return make_hostent(sock_addrinfo(host, Qnil, SOCK_STREAM, AI_CANONNAME), sock_sockaddr);
 }
 
 static VALUE
@@ -2074,6 +2006,8 @@ sock_s_gethostbyaddr(argc, argv)
     VALUE addr, type;
     struct hostent *h;
     struct sockaddr *sa;
+    char **pch;
+    VALUE ary, names;
     int t = AF_INET;
 
     rb_scan_args(argc, argv, "11", &addr, &type);
@@ -2086,9 +2020,34 @@ sock_s_gethostbyaddr(argc, argv)
 	t = AF_INET6;
     }
 #endif
-    h = gethostbyaddr(RSTRING(addr)->ptr, RSTRING(addr)->len, t);
+    h = gethostbyaddr((char*)RSTRING(addr)->ptr, RSTRING(addr)->len, t);
+    if (h == NULL) {
+#ifdef HAVE_HSTRERROR
+	extern int h_errno;
+	rb_raise(rb_eSocket, "%s", (char*)hstrerror(h_errno));
+#else
+	rb_raise(rb_eSocket, "host not found");
+#endif
+    }
+    ary = rb_ary_new();
+    rb_ary_push(ary, rb_str_new2(h->h_name));
+    names = rb_ary_new();
+    rb_ary_push(ary, names);
+    if (h->h_aliases != NULL) {
+	for (pch = h->h_aliases; *pch; pch++) {
+	    rb_ary_push(names, rb_str_new2(*pch));
+	}
+    }
+    rb_ary_push(ary, INT2NUM(h->h_addrtype));
+#ifdef h_addr
+    for (pch = h->h_addr_list; *pch; pch++) {
+	rb_ary_push(ary, rb_str_new(*pch, h->h_length));
+    }
+#else
+    rb_ary_push(ary, rb_str_new(h->h_addr, h->h_length));
+#endif
 
-    return sock_mkhostent(h);
+    return ary;
 }
 
 static VALUE
@@ -2106,7 +2065,7 @@ sock_s_getservbyaname(argc, argv)
     else proto = StringValuePtr(protocol);
 
     StringValue(service);
-    sp = getservbyname(RSTRING(service)->ptr, proto);
+    sp = getservbyname((char*)RSTRING(service)->ptr, proto);
     if (sp) {
 	port = ntohs(sp->s_port);
     }
@@ -2147,7 +2106,7 @@ sock_s_getaddrinfo(argc, argv)
 	pptr = NULL;
     }
     else if (FIXNUM_P(port)) {
-	snprintf(pbuf, sizeof(pbuf), "%d", FIX2INT(port));
+	snprintf(pbuf, sizeof(pbuf), "%ld", FIX2LONG(port));
 	pptr = pbuf;
     }
     else {
@@ -2188,7 +2147,7 @@ sock_s_getaddrinfo(argc, argv)
 	rb_raise(rb_eSocket, "getaddrinfo: %s", gai_strerror(error));
     }
 
-    ret = mkaddrinfo(res);
+    ret = make_addrinfo(res);
     freeaddrinfo(res);
     return ret;
 }
@@ -2266,7 +2225,7 @@ sock_s_getnameinfo(argc, argv)
 	    pptr = NULL;
 	}
 	else if (FIXNUM_P(port)) {
-	    snprintf(pbuf, sizeof(pbuf), "%d", NUM2INT(port));
+	    snprintf(pbuf, sizeof(pbuf), "%ld", NUM2LONG(port));
 	    pptr = pbuf;
 	}
 	else {
@@ -2351,7 +2310,7 @@ sock_s_unpack_sockaddr_in(self, addr)
     VALUE host;
 
     sockaddr = (struct sockaddr_in*)StringValuePtr(addr);
-    host = mkipaddr((struct sockaddr*)sockaddr);
+    host = make_ipaddr((struct sockaddr*)sockaddr);
     OBJ_INFECT(host, addr);
     return rb_assoc_new(INT2NUM(ntohs(sockaddr->sin_port)), host);
 }
@@ -2552,10 +2511,10 @@ Init_socket()
     sock_define_const("AF_UNSPEC", AF_UNSPEC);
     sock_define_const("PF_UNSPEC", PF_UNSPEC);
 #endif
-#ifdef AF_INET6
+#ifdef INET6
     sock_define_const("AF_INET6", AF_INET6);
 #endif
-#ifdef PF_INET6
+#ifdef INET6
     sock_define_const("PF_INET6", PF_INET6);
 #endif
 
