@@ -2,7 +2,7 @@
  * rubyext.c
  *
  * $Author: matz $
- * $Date: 2003/08/01 10:57:32 $
+ * $Date: 2003/12/21 15:38:01 $
  *
  * Copyright (C) 2003 why the lucky stiff
  */
@@ -35,15 +35,23 @@ typedef struct RVALUE {
     } as;
 } RVALUE;
 
+typedef struct {
+   long hash;
+   char *buffer;
+   long length;
+   long remaining;
+   int  printed;
+} bytestring_t;
+
 #define RUBY_DOMAIN   "ruby.yaml.org,2002"
 
 /*
  * symbols and constants
  */
-static ID s_new, s_utc, s_at, s_to_f, s_read, s_binmode, s_call, s_transfer, s_update, s_dup, s_match;
-static VALUE sym_model, sym_generic;
+static ID s_new, s_utc, s_at, s_to_f, s_to_i, s_read, s_binmode, s_call, s_transfer, s_update, s_dup, s_match, s_keys, s_to_str, s_unpack, s_tr_bang, s_anchors, s_default_set;
+static VALUE sym_model, sym_generic, sym_input, sym_bytecode;
 static VALUE sym_scalar, sym_seq, sym_map;
-VALUE cDate, cParser, cLoader, cNode, cPrivateType, cDomainType, cBadAlias, cMergeKey, cEmitter;
+VALUE cDate, cParser, cLoader, cNode, cPrivateType, cDomainType, cBadAlias, cDefaultKey, cMergeKey, cEmitter;
 VALUE oDefaultLoader;
 
 /*
@@ -64,12 +72,48 @@ SYMID rb_syck_load_handler _((SyckParser *, SyckNode *));
 void rb_syck_err_handler _((SyckParser *, char *));
 SyckNode * rb_syck_bad_anchor_handler _((SyckParser *, char *));
 void rb_syck_output_handler _((SyckEmitter *, char *, long));
+int syck_parser_assign_io _((SyckParser *, VALUE));
 
 struct parser_xtra {
     VALUE data;  /* Borrowed this idea from marshal.c to fix [ruby-core:8067] problem */
     VALUE proc;
+    int taint;
 };
 
+/*
+ * Convert YAML to bytecode
+ */
+VALUE
+rb_syck_compile(self, port)
+	VALUE self, port;
+{
+    SYMID oid;
+    int taint;
+    char *ret;
+    VALUE bc;
+    bytestring_t *sav; 
+
+    SyckParser *parser = syck_new_parser();
+	taint = syck_parser_assign_io(parser, port);
+    syck_parser_handler( parser, syck_yaml2byte_handler );
+    syck_parser_error_handler( parser, NULL );
+    syck_parser_implicit_typing( parser, 0 );
+    syck_parser_taguri_expansion( parser, 0 );
+    oid = syck_parse( parser );
+    syck_lookup_sym( parser, oid, (char **)&sav );
+
+    ret = S_ALLOC_N( char, strlen( sav->buffer ) + 3 );
+    ret[0] = '\0';
+    strcat( ret, "D\n" );
+    strcat( ret, sav->buffer );
+
+    syck_free_parser( parser );
+
+    bc = rb_str_new2( ret );
+    if ( taint )      OBJ_TAINT( bc );
+    return bc;
+}
+
 /*
  * read from io.
  */
@@ -80,20 +124,20 @@ rb_syck_io_str_read( char *buf, SyckIoStr *str, long max_size, long skip )
 
     ASSERT( str != NULL );
     max_size -= skip;
-    if ( max_size < 0 ) max_size = 0;
 
-    if ( max_size > 0 )
+    if ( max_size <= 0 ) max_size = 0;
+    else
     {
         /*
          * call io#read.
          */
         VALUE src = (VALUE)str->ptr;
         VALUE n = LONG2NUM(max_size);
-        VALUE str = rb_funcall2(src, s_read, 1, &n);
-        if (!NIL_P(str))
+        VALUE str2 = rb_funcall2(src, s_read, 1, &n);
+        if (!NIL_P(str2))
         {
-            len = RSTRING(str)->len;
-            memcpy( buf + skip, RSTRING(str)->ptr, len );
+            len = RSTRING(str2)->len;
+            memcpy( buf + skip, RSTRING(str2)->ptr, len );
         }
     }
     len += skip;
@@ -103,31 +147,29 @@ rb_syck_io_str_read( char *buf, SyckIoStr *str, long max_size, long skip )
 
 /*
  * determine: are we reading from a string or io?
+ * (returns tainted? boolean)
  */
-void
+int
 syck_parser_assign_io(parser, port)
 	SyckParser *parser;
 	VALUE port;
 {
-    if (rb_respond_to(port, rb_intern("to_str"))) {
-#if 0
-	    arg.taint = OBJ_TAINTED(port); /* original taintedness */
+    int taint = Qtrue;
+    if (rb_respond_to(port, s_to_str)) {
+	    taint = OBJ_TAINTED(port); /* original taintedness */
 	    StringValue(port);	       /* possible conversion */
-#endif
 	    syck_parser_str( parser, RSTRING(port)->ptr, RSTRING(port)->len, NULL );
     }
     else if (rb_respond_to(port, s_read)) {
         if (rb_respond_to(port, s_binmode)) {
             rb_funcall2(port, s_binmode, 0, 0);
         }
-#if 0
-        arg.taint = Qfalse;
-#endif
         syck_parser_str( parser, (char *)port, 0, rb_syck_io_str_read );
     }
     else {
         rb_raise(rb_eTypeError, "instance of IO needed");
     }
+    return taint;
 }
 
 /*
@@ -163,27 +205,27 @@ rb_syck_mktime(str)
 
     /* Month*/
     ptr += 4;
-    while ( !isdigit( *ptr ) ) ptr++;
+    while ( !ISDIGIT( *ptr ) ) ptr++;
     mon = INT2FIX(strtol(ptr, NULL, 10));
 
     /* Day*/
     ptr += 2;
-    while ( !isdigit( *ptr ) ) ptr++;
+    while ( !ISDIGIT( *ptr ) ) ptr++;
     day = INT2FIX(strtol(ptr, NULL, 10));
 
     /* Hour*/
     ptr += 2;
-    while ( !isdigit( *ptr ) ) ptr++;
+    while ( !ISDIGIT( *ptr ) ) ptr++;
     hour = INT2FIX(strtol(ptr, NULL, 10));
 
     /* Minute */
     ptr += 2;
-    while ( !isdigit( *ptr ) ) ptr++;
+    while ( !ISDIGIT( *ptr ) ) ptr++;
     min = INT2FIX(strtol(ptr, NULL, 10));
 
     /* Second */
     ptr += 2;
-    while ( !isdigit( *ptr ) ) ptr++;
+    while ( !ISDIGIT( *ptr ) ) ptr++;
     sec = INT2FIX(strtol(ptr, NULL, 10));
 
     /* Millisecond */
@@ -283,10 +325,8 @@ rb_syck_parse_handler(p, n)
     }
 
     bonus = (struct parser_xtra *)p->bonus;
-	if ( bonus->proc != 0 )
-	{
-		rb_funcall(bonus->proc, s_call, 1, v);
-	}
+    if ( bonus->taint)      OBJ_TAINT( obj );
+	if ( bonus->proc != 0 ) rb_funcall(bonus->proc, s_call, 1, v);
 
     rb_iv_set(obj, "@value", v);
     rb_hash_aset(bonus->data, INT2FIX(RHASH(bonus->data)->tbl->num_entries), obj);
@@ -309,79 +349,133 @@ syck_merge_i( entry, hsh )
 }
 
 /*
- * {native mode} node handler
- * - Converts data into native Ruby types
+ * build a syck node from a Ruby VALUE
  */
-SYMID
-rb_syck_load_handler(p, n)
-    SyckParser *p;
-    SyckNode *n;
+SyckNode *
+rb_new_syck_node( obj, type_id )
+    VALUE obj, type_id;
 {
+    long i = 0;
+    SyckNode *n = NULL;
+
+    if (rb_respond_to(obj, s_to_str)) 
+    {
+	    StringValue(obj);	       /* possible conversion */
+        n = syck_alloc_str();
+        n->data.str->ptr = RSTRING(obj)->ptr;
+        n->data.str->len = RSTRING(obj)->len;
+    }
+	else if ( rb_obj_is_kind_of( obj, rb_cArray ) )
+    {
+        n = syck_alloc_seq();
+        for ( i = 0; i < RARRAY(obj)->len; i++ )
+        {
+            syck_seq_add(n, rb_ary_entry(obj, i));
+        }
+    }
+    else if ( rb_obj_is_kind_of( obj, rb_cHash ) )
+    {
+        VALUE keys;
+        n = syck_alloc_map();
+        keys = rb_funcall( obj, s_keys, 0 );
+        for ( i = 0; i < RARRAY(keys)->len; i++ )
+        {
+            VALUE key = rb_ary_entry(keys, i);
+            syck_map_add(n, key, rb_hash_aref(obj, key));
+        }
+    }
+
+    if ( n!= NULL && rb_respond_to( type_id, s_to_str ) ) 
+    {
+        StringValue(type_id);
+        n->type_id = syck_strndup( RSTRING(type_id)->ptr, RSTRING(type_id)->len );
+    }
+
+    return n;
+}
+
+/*
+ * default handler for ruby.yaml.org types
+ */
+int
+yaml_org_handler( n, ref )
+    SyckNode *n;
+    VALUE *ref;
+{
+    char *type_id = n->type_id;
+    int transferred = 0;
+    long i = 0;
     VALUE obj = Qnil;
-    long i;
-    int check_transfers = 0;
-    struct parser_xtra *bonus;
 
     switch (n->kind)
     {
         case syck_str_kind:
-            if ( n->type_id == NULL || strcmp( n->type_id, "str" ) == 0 )
+            transferred = 1;
+            if ( type_id == NULL || strcmp( type_id, "str" ) == 0 )
             {
                 obj = rb_str_new( n->data.str->ptr, n->data.str->len );
             }
-            else if ( strcmp( n->type_id, "null" ) == 0 )
+            else if ( strcmp( type_id, "null" ) == 0 )
             {
                 obj = Qnil;
             }
-            else if ( strcmp( n->type_id, "bool#yes" ) == 0 )
+            else if ( strcmp( type_id, "binary" ) == 0 )
+            {
+                VALUE arr;
+                obj = rb_str_new( n->data.str->ptr, n->data.str->len );
+                rb_funcall( obj, s_tr_bang, 2, rb_str_new2( "\n\t " ), rb_str_new2( "" ) );
+                arr = rb_funcall( obj, s_unpack, 1, rb_str_new2( "m" ) );
+                obj = rb_ary_shift( arr );
+            }
+            else if ( strcmp( type_id, "bool#yes" ) == 0 )
             {
                 obj = Qtrue;
             }
-            else if ( strcmp( n->type_id, "bool#no" ) == 0 )
+            else if ( strcmp( type_id, "bool#no" ) == 0 )
             {
                 obj = Qfalse;
             }
-            else if ( strcmp( n->type_id, "int#hex" ) == 0 )
+            else if ( strcmp( type_id, "int#hex" ) == 0 )
             {
                 obj = rb_cstr2inum( n->data.str->ptr, 16 );
             }
-            else if ( strcmp( n->type_id, "int#oct" ) == 0 )
+            else if ( strcmp( type_id, "int#oct" ) == 0 )
             {
                 obj = rb_cstr2inum( n->data.str->ptr, 8 );
             }
-            else if ( strncmp( n->type_id, "int", 3 ) == 0 )
+            else if ( strncmp( type_id, "int", 3 ) == 0 )
             {
                 syck_str_blow_away_commas( n );
                 obj = rb_cstr2inum( n->data.str->ptr, 10 );
             }
-            else if ( strcmp( n->type_id, "float#nan" ) == 0 )
+            else if ( strcmp( type_id, "float#nan" ) == 0 )
             {
                 obj = rb_float_new( S_nan() );
             }
-            else if ( strcmp( n->type_id, "float#inf" ) == 0 )
+            else if ( strcmp( type_id, "float#inf" ) == 0 )
             {
                 obj = rb_float_new( S_inf() );
             }
-            else if ( strcmp( n->type_id, "float#neginf" ) == 0 )
+            else if ( strcmp( type_id, "float#neginf" ) == 0 )
             {
                 obj = rb_float_new( -S_inf() );
             }
-            else if ( strncmp( n->type_id, "float", 5 ) == 0 )
+            else if ( strncmp( type_id, "float", 5 ) == 0 )
             {
                 double f;
                 syck_str_blow_away_commas( n );
                 f = strtod( n->data.str->ptr, NULL );
                 obj = rb_float_new( f );
             }
-            else if ( strcmp( n->type_id, "timestamp#iso8601" ) == 0 )
+            else if ( strcmp( type_id, "timestamp#iso8601" ) == 0 )
             {
                 obj = rb_syck_mktime( n->data.str->ptr );
             }
-            else if ( strcmp( n->type_id, "timestamp#spaced" ) == 0 )
+            else if ( strcmp( type_id, "timestamp#spaced" ) == 0 )
             {
                 obj = rb_syck_mktime( n->data.str->ptr );
             }
-            else if ( strcmp( n->type_id, "timestamp#ymd" ) == 0 )
+            else if ( strcmp( type_id, "timestamp#ymd" ) == 0 )
             {
                 char *ptr = n->data.str->ptr;
                 VALUE year, mon, day;
@@ -392,47 +486,58 @@ rb_syck_load_handler(p, n)
 
                 /* Month*/
                 ptr += 4;
-                while ( !isdigit( *ptr ) ) ptr++;
+                while ( !ISDIGIT( *ptr ) ) ptr++;
                 mon = INT2FIX(strtol(ptr, NULL, 10));
 
                 /* Day*/
                 ptr += 2;
-                while ( !isdigit( *ptr ) ) ptr++;
+                while ( !ISDIGIT( *ptr ) ) ptr++;
                 day = INT2FIX(strtol(ptr, NULL, 10));
 
                 obj = rb_funcall( cDate, s_new, 3, year, mon, day );
             }
-            else if ( strncmp( n->type_id, "timestamp", 9 ) == 0 )
+            else if ( strncmp( type_id, "timestamp", 9 ) == 0 )
             {
                 obj = rb_syck_mktime( n->data.str->ptr );
             }
-			else if ( strncmp( n->type_id, "merge", 5 ) == 0 )
+			else if ( strncmp( type_id, "merge", 5 ) == 0 )
 			{
 				obj = rb_funcall( cMergeKey, s_new, 0 );
 			}
+			else if ( strncmp( type_id, "default", 7 ) == 0 )
+			{
+				obj = rb_funcall( cDefaultKey, s_new, 0 );
+			}
             else
             {
-                check_transfers = 1;
+                transferred = 0;
                 obj = rb_str_new( n->data.str->ptr, n->data.str->len );
             }
         break;
 
         case syck_seq_kind:
+            if ( type_id == NULL || strcmp( type_id, "seq" ) == 0 )
+            {
+                transferred = 1;
+            }
             obj = rb_ary_new2( n->data.list->idx );
             for ( i = 0; i < n->data.list->idx; i++ )
             {
                 rb_ary_store( obj, i, syck_seq_read( n, i ) );
             }
-            check_transfers = 1;
         break;
 
         case syck_map_kind:
+            if ( type_id == NULL || strcmp( type_id, "map" ) == 0 )
+            {
+                transferred = 1;
+            }
             obj = rb_hash_new();
             for ( i = 0; i < n->data.pairs->idx; i++ )
             {
 				VALUE k = syck_map_read( n, map_key, i );
 				VALUE v = syck_map_read( n, map_value, i );
-				int merge_key = 0;
+				int skip_aset = 0;
 
 				/*
 				 * Handle merge keys
@@ -444,7 +549,7 @@ rb_syck_load_handler(p, n)
 						VALUE dup = rb_funcall( v, s_dup, 0 );
 						rb_funcall( dup, s_update, 1, obj );
 						obj = dup;
-						merge_key = 1;
+						skip_aset = 1;
 					}
 					else if ( rb_obj_is_kind_of( v, rb_cArray ) )
 					{
@@ -456,20 +561,49 @@ rb_syck_load_handler(p, n)
 							rb_ary_push( v, obj );
 							rb_iterate( rb_each, v, syck_merge_i, dup );
 							obj = dup;
-							merge_key = 1;
+							skip_aset = 1;
 						}
 					}
 				}
+                else if ( rb_obj_is_kind_of( k, cDefaultKey ) )
+                {
+                    rb_funcall( obj, s_default_set, 1, v );
+                    skip_aset = 1;
+                }
 
-				if ( ! merge_key )
+				if ( ! skip_aset )
 				{
 					rb_hash_aset( obj, k, v );
 				}
             }
-            check_transfers = 1;
         break;
     }
 
+    *ref = obj;
+    return transferred;
+}
+
+/*
+ * {native mode} node handler
+ * - Converts data into native Ruby types
+ */
+SYMID
+rb_syck_load_handler(p, n)
+    SyckParser *p;
+    SyckNode *n;
+{
+    VALUE obj = Qnil;
+    struct parser_xtra *bonus;
+
+    /*
+     * Attempt common transfers
+     */
+    int transferred = yaml_org_handler(n, &obj);
+    if ( transferred == 0 && n->type_id != NULL )
+    {
+        obj = rb_funcall( oDefaultLoader, s_transfer, 2, rb_str_new2( n->type_id ), obj );
+    }
+
     /*
      * ID already set, let's alter the symbol table to accept the new object
      */
@@ -481,15 +615,8 @@ rb_syck_load_handler(p, n)
     }
 
     bonus = (struct parser_xtra *)p->bonus;
-	if ( bonus->proc != 0 )
-	{
-		rb_funcall(bonus->proc, s_call, 1, obj);
-	}
-
-    if ( check_transfers == 1 && n->type_id != NULL )
-    {
-        obj = rb_funcall( oDefaultLoader, s_transfer, 2, rb_str_new2( n->type_id ), obj );
-    }
+    if ( bonus->taint)      OBJ_TAINT( obj );
+	if ( bonus->proc != 0 ) rb_funcall(bonus->proc, s_call, 1, obj);
 
     rb_hash_aset(bonus->data, INT2FIX(RHASH(bonus->data)->tbl->num_entries), obj);
     return obj;
@@ -524,8 +651,9 @@ rb_syck_bad_anchor_handler(p, a)
     SyckParser *p;
     char *a;
 {
-    SyckNode *badanc = syck_new_map( rb_str_new2( "name" ), rb_str_new2( a ) );
-    badanc->type_id = syck_strndup( "taguri:ruby.yaml.org,2002:object:YAML::Syck::BadAlias", 53 );
+    VALUE anchor_name = rb_str_new2( a );
+    SyckNode *badanc = syck_new_map( rb_str_new2( "name" ), anchor_name );
+    badanc->type_id = syck_strndup( "tag:ruby.yaml.org,2002:object:YAML::Syck::BadAlias", 53 );
     return badanc;
 }
 
@@ -533,9 +661,9 @@ rb_syck_bad_anchor_handler(p, a)
  * data loaded based on the model requested.
  */
 void
-syck_set_model( parser, model )
+syck_set_model( parser, input, model )
 	SyckParser *parser;
-	VALUE model;
+	VALUE input, model;
 {
 	if ( model == sym_generic )
 	{
@@ -549,6 +677,10 @@ syck_set_model( parser, model )
 		syck_parser_implicit_typing( parser, 1 );
 		syck_parser_taguri_expansion( parser, 0 );
 	}
+    if ( input == sym_bytecode )
+    {
+        syck_parser_set_input_type( parser, syck_bytecode_utf8 );
+    }
     syck_parser_error_handler( parser, rb_syck_err_handler );
     syck_parser_bad_anchor_handler( parser, rb_syck_bad_anchor_handler );
 }
@@ -601,6 +733,35 @@ syck_parser_initialize( self, options )
 	return self;
 }
 
+/*
+ * YAML::Syck::Parser.bufsize = Integer
+ */
+static VALUE
+syck_parser_bufsize_set( self, size )
+    VALUE self, size;
+{
+	SyckParser *parser;
+
+	Data_Get_Struct(self, SyckParser, parser);
+    if ( rb_respond_to( size, s_to_i ) ) {
+        parser->bufsize = NUM2INT(rb_funcall(size, s_to_i, 0));
+    }
+	return self;
+}
+
+/*
+ * YAML::Syck::Parser.bufsize => Integer
+ */
+static VALUE
+syck_parser_bufsize_get( self )
+    VALUE self;
+{
+	SyckParser *parser;
+
+	Data_Get_Struct(self, SyckParser, parser);
+	return INT2FIX( parser->bufsize );
+}
+
 /*
  * YAML::Syck::Parser.load( IO or String )
  */
@@ -610,18 +771,19 @@ syck_parser_load(argc, argv, self)
     VALUE *argv;
 	VALUE self;
 {
-    VALUE port, proc, model;
+    VALUE port, proc, model, input;
 	SyckParser *parser;
     struct parser_xtra bonus;
     volatile VALUE hash;	/* protect from GC */
 
     rb_scan_args(argc, argv, "11", &port, &proc);
 	Data_Get_Struct(self, SyckParser, parser);
-	syck_parser_assign_io(parser, port);
 
+	input = rb_hash_aref( rb_iv_get( self, "@options" ), sym_input );
 	model = rb_hash_aref( rb_iv_get( self, "@options" ), sym_model );
-	syck_set_model( parser, model );
+	syck_set_model( parser, input, model );
 
+	bonus.taint = syck_parser_assign_io(parser, port);
     bonus.data = hash = rb_hash_new();
 	if ( NIL_P( proc ) ) bonus.proc = 0;
     else                 bonus.proc = proc;
@@ -640,18 +802,19 @@ syck_parser_load_documents(argc, argv, self)
     VALUE *argv;
 	VALUE self;
 {
-    VALUE port, proc, v, model;
+    VALUE port, proc, v, input, model;
 	SyckParser *parser;
     struct parser_xtra bonus;
     volatile VALUE hash;
 
     rb_scan_args(argc, argv, "1&", &port, &proc);
 	Data_Get_Struct(self, SyckParser, parser);
-	syck_parser_assign_io(parser, port);
 
+	input = rb_hash_aref( rb_iv_get( self, "@options" ), sym_input );
 	model = rb_hash_aref( rb_iv_get( self, "@options" ), sym_model );
-	syck_set_model( parser, model );
+	syck_set_model( parser, input, model );
     
+	bonus.taint = syck_parser_assign_io(parser, port);
     while ( 1 )
 	{
         /* Reset hash for tracking nodes */
@@ -682,14 +845,15 @@ syck_loader_initialize( self )
 {
     VALUE families;
 
-       rb_iv_set(self, "@families", rb_hash_new() );
+	rb_iv_set(self, "@families", rb_hash_new() );
     rb_iv_set(self, "@private_types", rb_hash_new() );
+    rb_iv_set(self, "@anchors", rb_hash_new() );
     families = rb_iv_get(self, "@families");
 
     rb_hash_aset(families, rb_str_new2( YAML_DOMAIN ), rb_hash_new());
     rb_hash_aset(families, rb_str_new2( RUBY_DOMAIN ), rb_hash_new());
 
-       return self;
+	return self;
 }
 
 /*
@@ -822,10 +986,6 @@ syck_loader_transfer( self, type, val )
 {
     char *taguri = NULL;
 
-#if 0
-	rb_p(rb_str_new2( "-- TYPE --" ));
-	rb_p(type);
-#endif
     if (NIL_P(type) || !RSTRING(type)->ptr || RSTRING(type)->len == 0) 
     {
         /*
@@ -833,6 +993,7 @@ syck_loader_transfer( self, type, val )
          */
         if ( TYPE(val) == T_STRING )
         {
+            StringValue(val);
             taguri = syck_match_implicit( RSTRING(val)->ptr, RSTRING(val)->len );
             taguri = syck_taguri( YAML_DOMAIN, taguri, strlen( taguri ) );
         }
@@ -844,14 +1005,13 @@ syck_loader_transfer( self, type, val )
 
     if ( taguri != NULL )
     {
+        int transferred = 0;
         VALUE scheme, name, type_hash, domain = Qnil, type_proc = Qnil;
         VALUE type_uri = rb_str_new2( taguri );
-        VALUE str_taguri = rb_str_new2("taguri");
+        VALUE str_taguri = rb_str_new2("tag");
         VALUE str_xprivate = rb_str_new2("x-private");
+        VALUE str_yaml_domain = rb_str_new2(YAML_DOMAIN);
         VALUE parts = rb_str_split( type_uri, ":" );
-#if 0
-        rb_p(parts);
-#endif
 
         scheme = rb_ary_shift( parts );
 
@@ -866,6 +1026,21 @@ syck_loader_transfer( self, type, val )
             name = rb_ary_join( parts, rb_str_new2( ":" ) );
             type_hash = rb_iv_get(self, "@families");
             type_hash = rb_hash_aref(type_hash, domain);
+
+            /*
+             * Route yaml.org types through the transfer
+             * method here in this extension
+             */
+            if ( rb_str_cmp( domain, str_yaml_domain ) == 0 )
+            {
+                SyckNode *n = rb_new_syck_node(val, name);
+                if ( n != NULL )
+                {
+                    transferred = yaml_org_handler(n, &val);
+                    S_FREE( n );
+                }
+            }
+
         }
         else
         {
@@ -873,34 +1048,34 @@ syck_loader_transfer( self, type, val )
                        scheme);
         }
 
-        if ( rb_obj_is_instance_of( type_hash, rb_cHash ) )
+        if ( ! transferred )
         {
-            type_proc = rb_hash_aref( type_hash, name );
-            if ( NIL_P( type_proc ) )
+            if ( rb_obj_is_instance_of( type_hash, rb_cHash ) )
             {
-                VALUE col = rb_ary_new();
-                rb_ary_push( col, name );
-                rb_iterate(rb_each, type_hash, transfer_find_i, col );
-                name = rb_ary_shift( col );
-                type_proc = rb_ary_shift( col );
+                type_proc = rb_hash_aref( type_hash, name );
+                if ( NIL_P( type_proc ) )
+                {
+                    VALUE col = rb_ary_new();
+                    rb_ary_push( col, name );
+                    rb_iterate(rb_each, type_hash, transfer_find_i, col );
+                    name = rb_ary_shift( col );
+                    type_proc = rb_ary_shift( col );
+                }
             }
-#if 0
-            rb_p(name);
-            rb_p(type_proc);
-#endif
-        }
 
-        if ( rb_respond_to( type_proc, s_call ) )
-        {
-            val = rb_funcall(type_proc, s_call, 2, type_uri, val);
-        }
-        else if ( rb_str_cmp( scheme, str_xprivate ) == 0 )
-        {
-            val = rb_funcall(cPrivateType, s_new, 2, name, val);
-        }
-        else
-        {
-            val = rb_funcall(cDomainType, s_new, 3, domain, name, val);
+            if ( rb_respond_to( type_proc, s_call ) )
+            {
+                val = rb_funcall(type_proc, s_call, 2, type_uri, val);
+            }
+            else if ( rb_str_cmp( scheme, str_xprivate ) == 0 )
+            {
+                val = rb_funcall(cPrivateType, s_new, 2, name, val);
+            }
+            else 
+            {
+                val = rb_funcall(cDomainType, s_new, 3, domain, name, val);
+            }
+            transferred = 1;
         }
     }
 
@@ -1012,7 +1187,7 @@ rb_syck_output_handler( emitter, str, len )
     long len;
 {
     VALUE dest = (VALUE)emitter->bonus;
-    if ( rb_respond_to( dest, rb_intern("to_str") ) ) {
+    if ( rb_respond_to( dest, s_to_str ) ) {
         rb_str_cat( dest, str, len );
     } else {
         rb_io_write( dest, rb_str_new( str, len ) );
@@ -1108,7 +1283,7 @@ syck_emitter_write_m( self, str )
 {
     SyckEmitter *emitter;
 
-    Data_Get_Struct(self, SyckEmitter, emitter);
+	Data_Get_Struct(self, SyckEmitter, emitter);
     syck_emitter_write( emitter, RSTRING(str)->ptr, RSTRING(str)->len );
     return self;
 }
@@ -1122,7 +1297,7 @@ syck_emitter_simple_write( self, str )
 {
     SyckEmitter *emitter;
 
-    Data_Get_Struct(self, SyckEmitter, emitter);
+	Data_Get_Struct(self, SyckEmitter, emitter);
     syck_emitter_simple( emitter, RSTRING(str)->ptr, RSTRING(str)->len );
     return self;
 }
@@ -1176,6 +1351,7 @@ Init_syck()
     VALUE rb_yaml = rb_define_module( "YAML" );
     VALUE rb_syck = rb_define_module_under( rb_yaml, "Syck" );
     rb_define_const( rb_syck, "VERSION", rb_str_new2( SYCK_VERSION ) );
+    rb_define_module_function( rb_syck, "compile", rb_syck_compile, 1 );
 
 	/*
 	 * Global symbols
@@ -1184,16 +1360,25 @@ Init_syck()
     s_utc = rb_intern("utc");
     s_at = rb_intern("at");
     s_to_f = rb_intern("to_f");
+    s_to_i = rb_intern("to_i");
     s_read = rb_intern("read");
+    s_anchors = rb_intern("anchors");
     s_binmode = rb_intern("binmode");
     s_transfer = rb_intern("transfer");
     s_call = rb_intern("call");
 	s_update = rb_intern("update");
 	s_dup = rb_intern("dup");
+    s_default_set = rb_intern("default=");
 	s_match = rb_intern("match");
+	s_keys = rb_intern("keys");
+	s_to_str = rb_intern("to_str");
+	s_tr_bang = rb_intern("tr!");
+    s_unpack = rb_intern("unpack");
 
 	sym_model = ID2SYM(rb_intern("Model"));
 	sym_generic = ID2SYM(rb_intern("Generic"));
+	sym_input = ID2SYM(rb_intern("Input"));
+	sym_bytecode = ID2SYM(rb_intern("Bytecode"));
     sym_map = ID2SYM(rb_intern("map"));
     sym_scalar = ID2SYM(rb_intern("scalar"));
     sym_seq = ID2SYM(rb_intern("seq"));
@@ -1210,11 +1395,14 @@ Init_syck()
     cLoader = rb_define_class_under( rb_syck, "Loader", rb_cObject );
     rb_define_attr( cLoader, "families", 1, 1 );
     rb_define_attr( cLoader, "private_types", 1, 1 );
+    rb_define_attr( cLoader, "anchors", 1, 1 );
     rb_define_method( cLoader, "initialize", syck_loader_initialize, 0 );
     rb_define_method( cLoader, "add_domain_type", syck_loader_add_domain_type, -1 );
     rb_define_method( cLoader, "add_builtin_type", syck_loader_add_builtin_type, -1 );
     rb_define_method( cLoader, "add_ruby_type", syck_loader_add_ruby_type, -1 );
     rb_define_method( cLoader, "add_private_type", syck_loader_add_private_type, -1 );
+    rb_define_method( cLoader, "bufsize=", syck_parser_bufsize_set, 1 );
+    rb_define_method( cLoader, "bufsize", syck_parser_bufsize_get, 0 );
     rb_define_method( cLoader, "detect_implicit", syck_loader_detect_implicit, 1 );
     rb_define_method( cLoader, "transfer", syck_loader_transfer, 2 );
 
@@ -1271,6 +1459,11 @@ Init_syck()
 	 */
 	cMergeKey = rb_define_class_under( rb_syck, "MergeKey", rb_cObject );
 
+	/*
+	 * Define YAML::Syck::DefaultKey class
+	 */
+	cDefaultKey = rb_define_class_under( rb_syck, "DefaultKey", rb_cObject );
+
     /*
      * Define YAML::Syck::Emitter class
      */
