@@ -1,8 +1,8 @@
 #
 #		tkcanvas.rb - Tk canvas classes
-#			$Date: 2003/08/03 08:53:06 $
+#			$Date: 2003/12/09 14:38:15 $
 #			by Yukihiro Matsumoto <matz@caelum.co.jp>
-#			$Date: 2003/08/03 08:53:06 $
+#			$Date: 2003/12/09 14:38:15 $
 #			by Hidetoshi Nagai <nagai@ai.kyutech.ac.jp>
 
 require "tk"
@@ -23,6 +23,8 @@ module TkTreatCItemFont
       self.path + ';' + tagOrId.to_s
     end
   end
+
+  private :__conf_cmd, :__item_pathname
 end
 
 class TkCanvas<TkWindow
@@ -44,6 +46,7 @@ class TkCanvas<TkWindow
       tk_call 'canvas', @path
     end
   end
+  private :create_self
 
   def tagid(tag)
     if tag.kind_of?(TkcItem) || tag.kind_of?(TkcTag)
@@ -211,6 +214,18 @@ class TkCanvas<TkWindow
       end
     when 'text', 'label', 'show', 'data', 'file', 'maskdata', 'maskfile'
       tk_send 'itemcget', tagid(tagOrId), "-#{option}"
+    when 'font', 'kanjifont'
+      #fnt = tk_tcl2ruby(tk_send('itemcget', tagid(tagOrId), "-#{option}"))
+      fnt = tk_tcl2ruby(tk_send('itemcget', tagid(tagOrId), '-font'))
+      unless fnt.kind_of?(TkFont)
+	fnt = tagfontobj(tagid(tagOrId), fnt)
+      end
+      if option.to_s == 'kanjifont' && JAPANIZED_TK && TK_VERSION =~ /^4\.*/
+	# obsolete; just for compatibility
+	fnt.kanji_font
+      else
+	fnt
+      end
     else
       tk_tcl2ruby tk_send('itemcget', tagid(tagOrId), "-#{option}")
     end
@@ -221,7 +236,7 @@ class TkCanvas<TkWindow
       key = _symbolkey2str(key)
       if ( key['font'] || key['kanjifont'] \
 	  || key['latinfont'] || key['asciifont'] )
-	tagfont_configure(tagOrId, key.dup)
+	tagfont_configure(tagid(tagOrId), key.dup)
       else
 	tk_send 'itemconfigure', tagid(tagOrId), *hash_kv(key)
       end
@@ -231,7 +246,11 @@ class TkCanvas<TkWindow
            key == 'kanjifont' || key == :kanjifont || 
 	   key == 'latinfont' || key == :latinfont || 
            key == 'asciifont' || key == :asciifont )
-	tagfont_configure(tagid(tagOrId), {key=>value})
+	if value == None
+	  tagfontobj(tagid(tagOrId))
+	else
+	  tagfont_configure(tagid(tagOrId), {key=>value})
+	end
       else
 	tk_send 'itemconfigure', tagid(tagOrId), "-#{key}", value
       end
@@ -264,6 +283,10 @@ class TkCanvas<TkWindow
       when 'text', 'label', 'show', 'data', 'file', 'maskdata', 'maskfile'
 	conf = tk_split_simplelist(tk_send('itemconfigure', 
 					   tagid(tagOrId), "-#{key}"))
+      when 'font', 'kanjifont'
+	conf = tk_split_simplelist(tk_send('itemconfigure',
+					   tagid(tagOrId),"-#{key}") )
+	conf[4] = tagfont_configinfo(tagid(tagOrId), conf[4])
       else
 	conf = tk_split_list(tk_send('itemconfigure', 
 				     tagid(tagOrId), "-#{key}"))
@@ -271,8 +294,8 @@ class TkCanvas<TkWindow
       conf[0] = conf[0][1..-1]
       conf
     else
-      tk_split_simplelist(tk_send('itemconfigure', 
-				  tagid(tagOrId))).collect{|conflist|
+      ret = tk_split_simplelist(tk_send('itemconfigure', 
+					tagid(tagOrId))).collect{|conflist|
 	conf = tk_split_simplelist(conflist)
 	conf[0] = conf[0][1..-1]
 	case conf[0]
@@ -302,6 +325,14 @@ class TkCanvas<TkWindow
 	end
 	conf
       }
+      fontconf = ret.assoc('font')
+      if fontconf
+	ret.delete_if{|item| item[0] == 'font' || item[0] == 'kanjifont'}
+	fontconf[4] = tagfont_configinfo(tagid(tagOrId), fontconf[4])
+	ret.push(fontconf)
+      else
+	ret
+      end
     end
   end
 
@@ -529,7 +560,7 @@ class TkcTag<TkObject
   include TkcTagAccess
 
   CTagID_TBL = TkCore::INTERP.create_table
-  Tk_CanvasTag_ID = ['ctag'.freeze, '00000'].freeze
+  Tk_CanvasTag_ID = ['ctag'.freeze, '00000'.taint].freeze
 
   TkCore::INTERP.init_ip_env{ CTagID_TBL.clear }
 
@@ -541,7 +572,7 @@ class TkcTag<TkObject
 
   def initialize(parent, mode=nil, *args)
     if not parent.kind_of?(TkCanvas)
-      fail format("%s need to be TkCanvas", parent.inspect)
+      fail Kernel.format("%s need to be TkCanvas", parent.inspect)
     end
     @c = parent
     @cpath = parent.path
@@ -619,7 +650,7 @@ class TkcTagString<TkcTag
 
   def initialize(parent, name, mode=nil, *args)
     if not parent.kind_of?(TkCanvas)
-      fail format("%s need to be TkCanvas", parent.inspect)
+      fail Kernel.format("%s need to be TkCanvas", parent.inspect)
     end
     @c = parent
     @cpath = parent.path
@@ -636,7 +667,7 @@ TkcNamedTag = TkcTagString
 class TkcTagAll<TkcTag
   def initialize(parent)
     if not parent.kind_of?(TkCanvas)
-      fail format("%s need to be TkCanvas", parent.inspect)
+      fail Kernel.format("%s need to be TkCanvas", parent.inspect)
     end
     @c = parent
     @cpath = parent.path
@@ -649,7 +680,7 @@ end
 class TkcTagCurrent<TkcTag
   def initialize(parent)
     if not parent.kind_of?(TkCanvas)
-      fail format("%s need to be TkCanvas", parent.inspect)
+      fail Kernel.format("%s need to be TkCanvas", parent.inspect)
     end
     @c = parent
     @cpath = parent.path
@@ -660,10 +691,10 @@ class TkcTagCurrent<TkcTag
 end
 
 class TkcGroup<TkcTag
-  Tk_cGroup_ID = ['tkcg'.freeze, '00000'].freeze
+  Tk_cGroup_ID = ['tkcg'.freeze, '00000'.taint].freeze
   def create_self(parent, *args)
     if not parent.kind_of?(TkCanvas)
-      fail format("%s need to be TkCanvas", parent.inspect)
+      fail Kernel.format("%s need to be TkCanvas", parent.inspect)
     end
     @c = parent
     @cpath = parent.path
@@ -673,6 +704,7 @@ class TkcGroup<TkcTag
     Tk_cGroup_ID[1].succ!
     add(*args) if args != []
   end
+  private :create_self
   
   def include(*tags)
     for i in tags
@@ -709,7 +741,7 @@ class TkcItem<TkObject
 
   def initialize(parent, *args)
     if not parent.kind_of?(TkCanvas)
-      fail format("%s need to be TkCanvas", parent.inspect)
+      fail Kernel.format("%s need to be TkCanvas", parent.inspect)
     end
     @parent = @c = parent
     @path = parent.path
@@ -732,7 +764,7 @@ class TkcItem<TkObject
     @id = create_self(*args).to_i ;# 'canvas item id' is integer number
     CItemID_TBL[@path] = {} unless CItemID_TBL[@path]
     CItemID_TBL[@path][@id] = self
-    font_configure(fontkeys) unless fontkeys.empty?
+    configure(fontkeys) unless fontkeys.empty?
 
 ######## old version
 #    if args[-1].kind_of? Hash
@@ -749,6 +781,7 @@ class TkcItem<TkObject
   end
   def create_self(*args); end
   private :create_self
+
   def id
     @id
   end
@@ -767,54 +800,71 @@ class TkcArc<TkcItem
   def create_self(*args)
     tk_call(@path, 'create', 'arc', *args)
   end
+  private :create_self
 end
+
 class TkcBitmap<TkcItem
   CItemTypeToClass['bitmap'] = self
   def create_self(*args)
     tk_call(@path, 'create', 'bitmap', *args)
   end
+  private :create_self
 end
+
 class TkcImage<TkcItem
   CItemTypeToClass['image'] = self
   def create_self(*args)
     tk_call(@path, 'create', 'image', *args)
   end
+  private :create_self
 end
+
 class TkcLine<TkcItem
   CItemTypeToClass['line'] = self
   def create_self(*args)
     tk_call(@path, 'create', 'line', *args)
   end
+  private :create_self
 end
+
 class TkcOval<TkcItem
   CItemTypeToClass['oval'] = self
   def create_self(*args)
     tk_call(@path, 'create', 'oval', *args)
   end
+  private :create_self
 end
+
 class TkcPolygon<TkcItem
   CItemTypeToClass['polygon'] = self
   def create_self(*args)
     tk_call(@path, 'create', 'polygon', *args)
   end
+  private :create_self
 end
+
 class TkcRectangle<TkcItem
   CItemTypeToClass['rectangle'] = self
   def create_self(*args)
     tk_call(@path, 'create', 'rectangle', *args)
   end
+  private :create_self
 end
+
 class TkcText<TkcItem
   CItemTypeToClass['text'] = self
   def create_self(*args)
     tk_call(@path, 'create', 'text', *args)
   end
+  private :create_self
 end
+
 class TkcWindow<TkcItem
   CItemTypeToClass['window'] = self
   def create_self(*args)
     tk_call(@path, 'create', 'window', *args)
   end
+  private :create_self
 end
 
 class TkImage<TkObject
@@ -823,7 +873,7 @@ class TkImage<TkObject
   TkCommandNames = ['image'.freeze].freeze
 
   Tk_IMGTBL = TkCore::INTERP.create_table
-  Tk_Image_ID = ['i'.freeze, '00000'].freeze
+  Tk_Image_ID = ['i'.freeze, '00000'.taint].freeze
 
   TkCore::INTERP.init_ip_env{ Tk_IMGTBL.clear }
 
