@@ -1,7 +1,7 @@
 #----------------------------------
 # extconf.rb
-# $Revision: 1.3 $
-# $Date: 2002/10/24 12:58:42 $
+# $Revision: 1.5 $
+# $Date: 2003/10/12 05:46:59 $
 #----------------------------------
 require 'mkmf'
 
@@ -10,7 +10,8 @@ def create_win32ole_makefile
      have_library("oleaut32") and
      have_library("uuid") and 
      have_library("user32") and
-     have_library("advapi32")
+     have_library("advapi32") and
+     have_header("windows.h")
     create_makefile("win32ole")
   end
 end
