@@ -3,7 +3,7 @@
  *
  *   Copyright (C) UENO Katsuhiro 2000-2003
  *
- * $Id: zlib.c,v 1.5 2003/07/20 02:06:15 katsu Exp $
+ * $Id: zlib.c,v 1.7 2003/12/24 15:19:11 matz Exp $
  */
 
 #include <ruby.h>
@@ -1813,7 +1813,7 @@ gzfile_read_all(gz)
 	if (!(gz->z.flags & GZFILE_FLAG_FOOTER_FINISHED)) {
 	    gzfile_check_footer(gz);
 	}
-	return Qnil;
+	return rb_str_new(0, 0);
     }
 
     dst = zstream_detach_buffer(&gz->z);
@@ -2749,6 +2749,7 @@ void Init_zlib()
     rb_define_method(cGzipReader, "gets", rb_gzreader_gets, -1);
     rb_define_method(cGzipReader, "readline", rb_gzreader_readline, -1);
     rb_define_method(cGzipReader, "each", rb_gzreader_each, -1);
+    rb_define_method(cGzipReader, "each_line", rb_gzreader_each, -1);
     rb_define_method(cGzipReader, "readlines", rb_gzreader_readlines, -1);
 
     rb_define_const(mZlib, "OS_CODE", INT2FIX(OS_CODE));
