@@ -2,8 +2,8 @@
 
   object.c -
 
-  $Author: matz $
-  $Date: 2003/07/24 18:33:40 $
+  $Author: nobu $
+  $Date: 2003/12/22 12:15:34 $
   created at: Thu Jul 15 12:01:24 JST 1993
 
   Copyright (C) 1993-2003 Yukihiro Matsumoto
@@ -476,6 +476,24 @@ true_xor(obj, obj2)
     return RTEST(obj2)?Qfalse:Qtrue;
 }
 
+
+/*
+ *  Document-class: FalseClass
+ *
+ *  The global value <code>false</code> is the only instance of class
+ *  <code>FalseClass</code> and represents a logically false value in
+ *  boolean expressions. The class provides operators allowing
+ *  <code>false</code> to participate correctly in logical expressions.
+ *     
+ */
+
+/*
+ * call-seq:
+ *   false.to_s   =>  "false"
+ *
+ * 'nuf said...
+ */
+
 static VALUE
 false_to_s(obj)
     VALUE obj;
@@ -483,6 +501,15 @@ false_to_s(obj)
     return rb_str_new2("false");
 }
 
+/*
+ *  call-seq:
+ *     false & obj   => false
+ *  
+ *  And---Returns <code>false</code>. <i>obj</i> is always
+ *  evaluated as it is the argument to a method call---there is no
+ *  short-circuit evaluation in this case.
+ */
+
 static VALUE
 false_and(obj, obj2)
     VALUE obj, obj2;
@@ -490,6 +517,15 @@ false_and(obj, obj2)
     return Qfalse;
 }
 
+
+/*
+ *  call-seq:
+ *     false | obj   =>   true or false
+ *  
+ *  Or---Returns <code>false</code> if <i>obj</i> is
+ *  <code>nil</code> or <code>false</code>; <code>true</code> otherwise.
+ */
+
 static VALUE
 false_or(obj, obj2)
     VALUE obj, obj2;
@@ -497,6 +533,18 @@ false_or(obj, obj2)
     return RTEST(obj2)?Qtrue:Qfalse;
 }
 
+
+
+/*
+ *  call-seq:
+ *     false ^ obj    => true or false
+ *  
+ *  Exclusive Or---If <i>obj</i> is <code>nil</code> or
+ *  <code>false</code>, returns <code>false</code>; otherwise, returns
+ *  <code>true</code>.
+ *     
+ */
+
 static VALUE
 false_xor(obj, obj2)
     VALUE obj, obj2;
@@ -675,25 +723,6 @@ rb_mod_cmp(mod, arg)
     return INT2FIX(1);
 }
 
-static VALUE
-rb_mod_initialize(module)
-    VALUE module;
-{
-    if (rb_block_given_p()) {
-	rb_mod_module_eval(0, 0, module);
-    }
-    return Qnil;
-}
-
-static VALUE
-rb_class_initialize(argc, argv, klass)
-    int argc;
-    VALUE *argv;
-    VALUE klass;
-{
-    return rb_mod_initialize(klass);
-}
-
 static VALUE rb_module_s_alloc _((VALUE));
 static VALUE
 rb_module_s_alloc(klass)
@@ -705,30 +734,77 @@ rb_module_s_alloc(klass)
     return mod;
 }
 
+static VALUE rb_class_s_alloc _((VALUE));
 static VALUE
-rb_class_s_new(argc, argv)
+rb_class_s_alloc(klass)
+    VALUE klass;
+{
+    return rb_class_boot(0);
+}
+
+static VALUE
+rb_mod_initialize(module)
+    VALUE module;
+{
+    if (rb_block_given_p()) {
+	rb_mod_module_eval(0, 0, module);
+    }
+    return Qnil;
+}
+
+/*
+ *  call-seq:
+ *     Class.new(super_class=Object)   =>    a_class
+ *  
+ *  Creates a new anonymous (unnamed) class with the given superclass
+ *  (or <code>Object</code> if no parameter is given). You can give a
+ *  class a name by assigning the class object to a constant.
+ *     
+ */
+
+static VALUE
+rb_class_initialize(argc, argv, klass)
     int argc;
     VALUE *argv;
+    VALUE klass;
 {
-    VALUE super, klass;
+    VALUE super;
 
+    if (RCLASS(klass)->super != 0) {
+	rb_raise(rb_eTypeError, "already initialized class");
+    }
     if (rb_scan_args(argc, argv, "01", &super) == 0) {
 	super = rb_cObject;
     }
-    klass = rb_class_new(super);
+    else {
+	rb_check_inheritable(super);
+    }
+    RCLASS(klass)->super = super;
     rb_make_metaclass(klass, RBASIC(super)->klass);
-    rb_obj_call_init(klass, argc, argv);
+    rb_mod_initialize(klass);
     rb_class_inherited(super, klass);
 
     return klass;
 }
 
+/*
+ *  call-seq:
+ *     class.allocate()   =>   obj
+ *  
+ *  Allocates space for a new object of <i>class</i>'s class. The
+ *  returned object must be an instance of <i>class</i>.
+ *     
+ */
+
 VALUE
 rb_obj_alloc(klass)
     VALUE klass;
 {
     VALUE obj;
 
+    if (RCLASS(klass)->super == 0) {
+	rb_raise(rb_eTypeError, "can't instantiate uninitialized class");
+    }
     if (FL_TEST(klass, FL_SINGLETON)) {
 	rb_raise(rb_eTypeError, "can't create instance of virtual class");
     }
@@ -749,6 +825,18 @@ rb_class_allocate_instance(klass)
     return (VALUE)obj;
 }
 
+/*
+ *  call-seq:
+ *     class.new(args, ...)    =>  obj
+ *  
+ *  Calls <code>allocate</code> to create a new object of
+ *  <i>class</i>'s class, then invokes that object's
+ *  <code>initialize</code> method, passing it <i>args</i>.
+ *  This is the method that ends up getting called whenever
+ *  an object is constructed using .new.
+ *     
+ */
+
 VALUE
 rb_class_new_instance(argc, argv, klass)
     int argc;
@@ -763,12 +851,27 @@ rb_class_new_instance(argc, argv, klass)
     return obj;
 }
 
+/*
+ *  call-seq:
+ *     class.superclass -> a_super_class or nil
+ *  
+ *  Returns the superclass of <i>class</i>, or <code>nil</code>.
+ *     
+ *     File.superclass     #=> IO
+ *     IO.superclass       #=> Object
+ *     Object.superclass   #=> nil
+ *     
+ */
+
 static VALUE
 rb_class_superclass(klass)
     VALUE klass;
 {
     VALUE super = RCLASS(klass)->super;
 
+    if (!super) {
+	rb_raise(rb_eTypeError, "uninitialized class");
+    }
     while (TYPE(super) == T_ICLASS) {
 	super = RCLASS(super)->super;
     }
@@ -1354,6 +1457,65 @@ boot_defclass(name, super)
 
 VALUE ruby_top_self;
 
+/*
+ *  Document-class: Class
+ *
+ *  Classes in Ruby are first-class objects---each is an instance of
+ *  class <code>Class</code>.
+ *     
+ *  When a new class is created (typically using <code>class Name ...
+ *  end</code>), an object of type <code>Class</code> is created and
+ *  assigned to a global constant (<code>Name</code> in this case). When
+ *  <code>Name.new</code> is called to create a new object, the
+ *  <code>new</code> method in <code>Class</code> is run by default.
+ *  This can be demonstrated by overriding <code>new</code> in
+ *  <code>Class</code>:
+ *     
+ *     class Class
+ *        alias oldNew  new
+ *        def new(*args)
+ *          print "Creating a new ", self.name, "\n"
+ *          oldNew(*args)
+ *        end
+ *      end
+ *     
+ *     
+ *      class Name
+ *      end
+ *     
+ *     
+ *      n = Name.new
+ *     
+ *  <em>produces:</em>
+ *     
+ *     Creating a new Name
+ *     
+ * Classes, modules, and objects are interrelated. In the diagram
+ * that follows, the arrows represent inheritance, and the
+ * parentheses meta-classes. All metaclasses are instances 
+ * of the class `Class'.
+ *
+ *                           +------------------+
+ *                           |                  |
+ *             Object---->(Object)              |
+ *              ^  ^        ^  ^                |
+ *              |  |        |  |                |
+ *              |  |  +-----+  +---------+      |
+ *              |  |  |                  |      |
+ *              |  +-----------+         |      |
+ *              |     |        |         |      |
+ *       +------+     |     Module--->(Module)  |
+ *       |            |        ^         ^      |
+ *  OtherClass-->(OtherClass)  |         |      |
+ *                             |         |      |
+ *                           Class---->(Class)  |
+ *                             ^                |
+ *                             |                |
+ *                             +----------------+
+ *
+ *
+ */
+
 void
 Init_Object()
 {
@@ -1373,33 +1535,11 @@ Init_Object()
     rb_define_private_method(rb_cObject, "initialize", rb_obj_dummy, 0);
     rb_define_private_method(rb_cClass, "inherited", rb_obj_dummy, 1);
     rb_define_private_method(rb_cModule, "included", rb_obj_dummy, 1);
+    rb_define_private_method(rb_cModule, "extended", rb_obj_dummy, 1);
     rb_define_private_method(rb_cModule, "method_added", rb_obj_dummy, 1);
     rb_define_private_method(rb_cModule, "method_removed", rb_obj_dummy, 1);
     rb_define_private_method(rb_cModule, "method_undefined", rb_obj_dummy, 1);
 
-    /*
-     * Ruby's Class Hierarchy Chart
-     *
-     *                           +------------------+
-     *                           |                  |
-     *             Object---->(Object)              |
-     *              ^  ^        ^  ^                |
-     *              |  |        |  |                |
-     *              |  |  +-----+  +---------+      |
-     *              |  |  |                  |      |
-     *              |  +-----------+         |      |
-     *              |     |        |         |      |
-     *       +------+     |     Module--->(Module)  |
-     *       |            |        ^         ^      |
-     *  OtherClass-->(OtherClass)  |         |      |
-     *                             |         |      |
-     *                           Class---->(Class)  |
-     *                             ^                |
-     *                             |                |
-     *                             +----------------+
-     *
-     *   + All metaclasses are instances of the class `Class'.
-     */
 
     rb_define_method(rb_mKernel, "nil?", rb_false, 0);
     rb_define_method(rb_mKernel, "==", rb_obj_equal, 1);
@@ -1491,8 +1631,7 @@ Init_Object()
     rb_define_method(rb_cModule, "<=", rb_mod_le, 1);
     rb_define_method(rb_cModule, ">",  rb_mod_gt, 1);
     rb_define_method(rb_cModule, ">=", rb_mod_ge, 1);
-    rb_define_method(rb_cModule, "clone", rb_mod_clone, 0);
-    rb_define_method(rb_cModule, "dup", rb_mod_dup, 0);
+    rb_define_method(rb_cModule, "initialize_copy", rb_mod_init_copy, 1);
     rb_define_method(rb_cModule, "to_s", rb_mod_to_s, 0);
     rb_define_method(rb_cModule, "included_modules", rb_mod_included_modules, 0);
     rb_define_method(rb_cModule, "include?", rb_mod_include_p, 1);
@@ -1523,9 +1662,9 @@ Init_Object()
     rb_define_method(rb_cClass, "allocate", rb_obj_alloc, 0);
     rb_define_method(rb_cClass, "new", rb_class_new_instance, -1);
     rb_define_method(rb_cClass, "initialize", rb_class_initialize, -1);
+    rb_define_method(rb_cClass, "initialize_copy", rb_class_init_copy, 1);
     rb_define_method(rb_cClass, "superclass", rb_class_superclass, 0);
-    rb_undef_alloc_func(rb_cClass);
-    rb_define_singleton_method(rb_cClass, "new", rb_class_s_new, -1);
+    rb_define_alloc_func(rb_cClass, rb_class_s_alloc);
     rb_undef_method(rb_cClass, "extend_object");
     rb_undef_method(rb_cClass, "append_features");
 
