@@ -3,7 +3,7 @@
   pack.c -
 
   $Author: matz $
-  $Date: 2003/07/09 22:28:36 $
+  $Date: 2003/12/23 19:53:42 $
   created at: Thu Feb 10 15:17:05 JST 1994
 
   Copyright (C) 1993-2003 Yukihiro Matsumoto
@@ -22,19 +22,25 @@
 #endif
 
 #ifdef NATINT_PACK
-# define OFF16B(p) ((char*)(p) + (natint?0:(sizeof(short) - SIZE16)))
-# define OFF32B(p) ((char*)(p) + (natint?0:(sizeof(long) - SIZE32)))
 # define NATINT_I32(x) (natint?NUM2LONG(x):(NUM2I32(x)))
 # define NATINT_U32(x) (natint?NUM2ULONG(x):(NUM2U32(x)))
 # define NATINT_LEN(type,len) (natint?sizeof(type):(len))
 # ifdef WORDS_BIGENDIAN
-#   define OFF16(p) OFF16B(p)
-#   define OFF32(p) OFF32B(p)
+#   define OFF16(p) ((char*)(p) + (natint?0:(sizeof(short) - SIZE16)))
+#   define OFF32(p) ((char*)(p) + (natint?0:(sizeof(long) - SIZE32)))
 # endif
+# define NATINT_HTOVS(x) (natint?htovs(x):htov16(x))
+# define NATINT_HTOVL(x) (natint?htovl(x):htov32(x))
+# define NATINT_HTONS(x) (natint?htons(x):hton16(x))
+# define NATINT_HTONL(x) (natint?htonl(x):hton32(x))
 #else
 # define NATINT_I32(x) NUM2I32(x)
 # define NATINT_U32(x) NUM2U32(x)
 # define NATINT_LEN(type,len) sizeof(type)
+# define NATINT_HTOVS(x) htovs(x)
+# define NATINT_HTOVL(x) htovl(x)
+# define NATINT_HTONS(x) htons(x)
+# define NATINT_HTONL(x) htonl(x)
 #endif
 
 #ifndef OFF16
@@ -42,11 +48,6 @@
 # define OFF32(p) (char*)(p)
 #endif
 
-#ifndef OFF16B
-# define OFF16B(p) (char*)(p)
-# define OFF32B(p) (char*)(p)
-#endif
-
 #define define_swapx(x, xtype)		\
 static xtype				\
 TOKEN_PASTE(swap,x)(z)			\
@@ -70,8 +71,11 @@ TOKEN_PASTE(swap,x)(z)			\
     return r;				\
 }
 
+#ifndef swap16
+#define swap16(x)	((((x)&0xFF)<<8) | (((x)>>8)&0xFF))
+#endif
 #if SIZEOF_SHORT == 2
-#define swaps(x)	((((x)&0xFF)<<8) | (((x)>>8)&0xFF))
+#define swaps(x)	swap16(x)
 #else
 #if SIZEOF_SHORT == 4
 #define swaps(x)	((((x)&0xFF)<<24)	\
@@ -79,15 +83,18 @@ TOKEN_PASTE(swap,x)(z)			\
 			|(((x)&0x0000FF00)<<8)	\
 			|(((x)&0x00FF0000)>>8)	)
 #else
-define_swapx(s,short);
+define_swapx(s,short)
 #endif
 #endif
 
-#if SIZEOF_LONG == 4
-#define swapl(x)	((((x)&0xFF)<<24)	\
+#ifndef swap32
+#define swap32(x)	((((x)&0xFF)<<24)	\
 			|(((x)>>24)&0xFF)	\
 			|(((x)&0x0000FF00)<<8)	\
 			|(((x)&0x00FF0000)>>8)	)
+#endif
+#if SIZEOF_LONG == 4
+#define swapl(x)	swap32(x)
 #else
 #if SIZEOF_LONG == 8
 #define swapl(x)        ((((x)&0x00000000000000FF)<<56)	\
@@ -99,7 +106,7 @@ define_swapx(s,short);
 			|(((x)&0x00000000FF000000)<<8)	\
 			|(((x)&0x000000FF00000000)>>8))
 #else
-define_swapx(l,long);
+define_swapx(l,long)
 #endif
 #endif
 
@@ -112,11 +119,11 @@ define_swapx(l,long);
 #define swapf(x)	swaps(x)
 #define FLOAT_SWAPPER	unsigned short
 #else	/* SIZEOF_FLOAT == 4 but undivide by known size of int */
-define_swapx(f,float);
+define_swapx(f,float)
 #endif	/* #if SIZEOF_SHORT == 4 */
 #endif	/* #if SIZEOF_LONG == 4 */
 #else	/* SIZEOF_FLOAT != 4 */
-define_swapx(f,float);
+define_swapx(f,float)
 #endif	/* #if SIZEOF_FLOAT == 4 */
 
 #if SIZEOF_DOUBLE == 8
@@ -160,12 +167,12 @@ swapd(d)
     return dtmp;
 }
 #else	/* SIZEOF_DOUBLE == 8 but undivied by known size of int */
-define_swapx(d, double);
+define_swapx(d, double)
 #endif	/* #if SIZEOF_SHORT == 4 */
 #endif	/* #if SIZEOF_LONG == 4 */
 #endif	/* #if SIZEOF_LONG == 8 */
 #else	/* SIZEOF_DOUBLE != 8 */
-define_swapx(d, double);
+define_swapx(d, double)
 #endif	/* #if SIZEOF_DPOUBLE == 8 */
 
 #undef define_swapx
@@ -206,6 +213,12 @@ endian()
 #define vtohl(x) (endian()?swapl(x):(x))
 #define vtohf(x) (endian()?swapf(x):(x))
 #define vtohd(x) (endian()?swapd(x):(x))
+# ifdef NATINT_PACK
+#define htov16(x) (endian()?swap16(x):(x))
+#define htov32(x) (endian()?swap32(x):(x))
+#define hton16(x) (endian()?(x):swap16(x))
+#define hton32(x) (endian()?(x):swap32(x))
+# endif
 #else
 #ifdef WORDS_BIGENDIAN
 #ifndef ntohs
@@ -226,6 +239,12 @@ endian()
 #define vtohl(x) swapl(x)
 #define vtohf(x) swapf(x)
 #define vtohd(x) swapd(x)
+# ifdef NATINT_PACK
+#define htov16(x) swap16(x)
+#define htov32(x) swap32(x)
+#define hton16(x) (x)
+#define hton32(x) (x)
+# endif
 #else /* LITTLE ENDIAN */
 #ifndef ntohs
 #undef ntohs
@@ -249,6 +268,12 @@ endian()
 #define vtohl(x) (x)
 #define vtohf(x) (x)
 #define vtohd(x) (x)
+# ifdef NATINT_PACK
+#define htov16(x) (x)
+#define htov32(x) (x)
+#define hton16(x) swap16(x)
+#define hton32(x) swap32(x)
+# endif
 #endif
 #endif
 
@@ -376,6 +401,71 @@ static void qpencode _((VALUE,VALUE,long));
 static int uv_to_utf8 _((char*,unsigned long));
 static unsigned long utf8_to_uv _((char*,long*));
 
+/*
+ *  call-seq:
+ *     arr.pack ( aTemplateString ) -> aBinaryString
+ *  
+ *  Packs the contents of <i>arr</i> into a binary sequence according to
+ *  the directives in <i>aTemplateString</i> (see the table below)
+ *  Directives ``A,'' ``a,'' and ``Z'' may be followed by a count,
+ *  which gives the width of the resulting field. The remaining
+ *  directives also may take a count, indicating the number of array
+ *  elements to convert. If the count is an asterisk
+ *  (``<code>*</code>''), all remaining array elements will be
+ *  converted. Any of the directives ``<code>sSiIlL</code>'' may be
+ *  followed by an underscore (``<code>_</code>'') to use the underlying
+ *  platform's native size for the specified type; otherwise, they use a
+ *  platform-independent size. Spaces are ignored in the template
+ *  string. See also <code>String#unpack</code>.
+ *     
+ *     a = [ "a", "b", "c" ]
+ *     n = [ 65, 66, 67 ]
+ *     a.pack("A3A3A3")   #=> "a  b  c  "
+ *     a.pack("a3a3a3")   #=> "a\000\000b\000\000c\000\000"
+ *     n.pack("ccc")      #=> "ABC"
+ *     
+ * Directives for +pack+.
+ *
+ *   Directive    Meaning
+ *   ---------------------------------------------------------------
+ *       @     |  Moves to absolute position
+ *       A     |  ASCII string (space padded, count is width)
+ *       a     |  ASCII string (null padded, count is width)
+ *       B     |  Bit string (descending bit order)
+ *       b     |  Bit string (ascending bit order)
+ *       C     |  Unsigned char
+ *       c     |  Char
+ *       D, d  |  Double-precision float, native format
+ *       E     |  Double-precision float, little-endian byte order
+ *       e     |  Single-precision float, little-endian byte order
+ *       F, f  |  Single-precision float, native format
+ *       G     |  Double-precision float, network (big-endian) byte order
+ *       g     |  Single-precision float, network (big-endian) byte order
+ *       H     |  Hex string (high nibble first)
+ *       h     |  Hex string (low nibble first)
+ *       I     |  Unsigned integer
+ *       i     |  Integer
+ *       L     |  Unsigned long
+ *       l     |  Long
+ *       M     |  Quoted printable, MIME encoding (see RFC2045)
+ *       m     |  Base64 encoded string
+ *       N     |  Long, network (big-endian) byte order
+ *       n     |  Short, network (big-endian) byte-order
+ *       P     |  Pointer to a structure (fixed-length string)
+ *       p     |  Pointer to a null-terminated string
+ *       Q, q  |  64-bit number
+ *       S     |  Unsigned short
+ *       s     |  Short
+ *       U     |  UTF-8
+ *       u     |  UU-encoded string
+ *       V     |  Long, little-endian byte order
+ *       v     |  Short, little-endian byte order
+ *       w     |  BER-compressed integer\fnm
+ *       X     |  Back up a byte
+ *       x     |  Null byte
+ *       Z     |  Same as ``A''
+ */
+
 static VALUE
 pack_pack(ary, fmt)
     VALUE ary, fmt;
@@ -459,9 +549,9 @@ pack_pack(ary, fmt)
 		len = plen;
 
 	    switch (type) {
-	      case 'a':
-	      case 'A':
-	      case 'Z':
+	      case 'a':		/* arbitrary binary string (null padded)  */
+	      case 'A':		/* ASCII string (space padded) */
+	      case 'Z':		/* null terminated ASCII string  */
 		if (plen >= len)
 		    rb_str_buf_cat(res, ptr, len);
 		else {
@@ -475,7 +565,7 @@ pack_pack(ary, fmt)
 		}
 		break;
 
-	      case 'b':
+	      case 'b':		/* bit string (ascending) */
 		{
 		    int byte = 0;
 		    long i, j = 0;
@@ -506,7 +596,7 @@ pack_pack(ary, fmt)
 		}
 		break;
 
-	      case 'B':
+	      case 'B':		/* bit string (descending) */
 		{
 		    int byte = 0;
 		    long i, j = 0;
@@ -536,7 +626,7 @@ pack_pack(ary, fmt)
 		}
 		break;
 
-	      case 'h':
+	      case 'h':		/* hex string (low nibble first) */
 		{
 		    int byte = 0;
 		    long i, j = 0;
@@ -567,7 +657,7 @@ pack_pack(ary, fmt)
 		}
 		break;
 
-	      case 'H':
+	      case 'H':		/* hex string (high nibble first) */
 		{
 		    int byte = 0;
 		    long i, j = 0;
@@ -600,8 +690,8 @@ pack_pack(ary, fmt)
 	    }
 	    break;
 
-	  case 'c':
-	  case 'C':
+	  case 'c':		/* signed char */
+	  case 'C':		/* unsigned char */
 	    while (len-- > 0) {
 		char c;
 
@@ -614,8 +704,8 @@ pack_pack(ary, fmt)
 	    }
 	    break;
 
-	  case 's':
-	  case 'S':
+	  case 's':		/* signed short */
+	  case 'S':		/* unsigned short */
 	    while (len-- > 0) {
 		short s;
 
@@ -628,22 +718,22 @@ pack_pack(ary, fmt)
 	    }
 	    break;
 
-	  case 'i':
-	  case 'I':
+	  case 'i':		/* signed int */
+	  case 'I':		/* unsigned int */
 	    while (len-- > 0) {
-		int i;
+		long i;
 
 		from = NEXTFROM;
 		if (NIL_P(from)) i = 0;
 		else {
-		    i = NUM2UINT(from);
+		    i = NATINT_I32(from);
 		}
-		rb_str_buf_cat(res, (char*)&i, sizeof(int));
+		rb_str_buf_cat(res, OFF32(&i), NATINT_LEN(int,4));
 	    }
 	    break;
 
-	  case 'l':
-	  case 'L':
+	  case 'l':		/* signed long */
+	  case 'L':		/* unsigned long */
 	    while (len-- > 0) {
 		long l;
 
@@ -656,8 +746,8 @@ pack_pack(ary, fmt)
 	    }
 	    break;
 
-	  case 'q':
-	  case 'Q':
+	  case 'q':		/* signed quad (64bit) int */
+	  case 'Q':		/* unsigned quad (64bit) int */
 	    while (len-- > 0) {
 		char tmp[QUAD_SIZE];
 
@@ -668,7 +758,7 @@ pack_pack(ary, fmt)
 	    }
 	    break;
 
-	  case 'n':
+	  case 'n':		/* unsigned short (network byte-order)  */
 	    while (len-- > 0) {
 		unsigned short s;
 
@@ -677,40 +767,12 @@ pack_pack(ary, fmt)
 		else {
 		    s = NUM2INT(from);
 		}
-		s = htons(s);
-		rb_str_buf_cat(res, OFF16B(&s), NATINT_LEN(short,2));
-	    }
-	    break;
-
-	  case 'N':
-	    while (len-- > 0) {
-		unsigned long l;
-
-		from = NEXTFROM;
-		if (NIL_P(from)) l = 0;
-		else {
-		    l = NATINT_U32(from);
-		}
-		l = htonl(l);
-		rb_str_buf_cat(res, OFF32B(&l), NATINT_LEN(long,4));
-	    }
-	    break;
-
-	  case 'v':
-	    while (len-- > 0) {
-		unsigned short s;
-
-		from = NEXTFROM;
-		if (NIL_P(from)) s = 0;
-		else {
-		    s = NUM2INT(from);
-		}
-		s = htovs(s);
+		s = NATINT_HTONS(s);
 		rb_str_buf_cat(res, OFF16(&s), NATINT_LEN(short,2));
 	    }
 	    break;
 
-	  case 'V':
+	  case 'N':		/* unsigned long (network byte-order) */
 	    while (len-- > 0) {
 		unsigned long l;
 
@@ -719,13 +781,41 @@ pack_pack(ary, fmt)
 		else {
 		    l = NATINT_U32(from);
 		}
-		l = htovl(l);
+		l = NATINT_HTONL(l);
 		rb_str_buf_cat(res, OFF32(&l), NATINT_LEN(long,4));
 	    }
 	    break;
 
-	  case 'f':
-	  case 'F':
+	  case 'v':		/* unsigned short (VAX byte-order) */
+	    while (len-- > 0) {
+		unsigned short s;
+
+		from = NEXTFROM;
+		if (NIL_P(from)) s = 0;
+		else {
+		    s = NUM2INT(from);
+		}
+		s = NATINT_HTOVS(s);
+		rb_str_buf_cat(res, OFF16(&s), NATINT_LEN(short,2));
+	    }
+	    break;
+
+	  case 'V':		/* unsigned long (VAX byte-order) */
+	    while (len-- > 0) {
+		unsigned long l;
+
+		from = NEXTFROM;
+		if (NIL_P(from)) l = 0;
+		else {
+		    l = NATINT_U32(from);
+		}
+		l = NATINT_HTOVL(l);
+		rb_str_buf_cat(res, OFF32(&l), NATINT_LEN(long,4));
+	    }
+	    break;
+
+	  case 'f':		/* single precision float in native format */
+	  case 'F':		/* ditto */
 	    while (len-- > 0) {
 		float f;
 
@@ -735,7 +825,7 @@ pack_pack(ary, fmt)
 	    }
 	    break;
 
-	  case 'e':
+	  case 'e':		/* single precision float in VAX byte-order */
 	    while (len-- > 0) {
 		float f;
 		FLOAT_CONVWITH(ftmp);
@@ -747,7 +837,7 @@ pack_pack(ary, fmt)
 	    }
 	    break;
 
-	  case 'E':
+	  case 'E':		/* double precision float in VAX byte-order */
 	    while (len-- > 0) {
 		double d;
 		DOUBLE_CONVWITH(dtmp);
@@ -759,8 +849,8 @@ pack_pack(ary, fmt)
 	    }
 	    break;
 
-	  case 'd':
-	  case 'D':
+	  case 'd':		/* double precision float in native format */
+	  case 'D':		/* ditto */
 	    while (len-- > 0) {
 		double d;
 
@@ -770,7 +860,7 @@ pack_pack(ary, fmt)
 	    }
 	    break;
 
-	  case 'g':
+	  case 'g':		/* single precision float in network byte-order */
 	    while (len-- > 0) {
 		float f;
 		FLOAT_CONVWITH(ftmp);
@@ -782,7 +872,7 @@ pack_pack(ary, fmt)
 	    }
 	    break;
 
-	  case 'G':
+	  case 'G':		/* double precision float in network byte-order */
 	    while (len-- > 0) {
 		double d;
 		DOUBLE_CONVWITH(dtmp);
@@ -794,7 +884,7 @@ pack_pack(ary, fmt)
 	    }
 	    break;
 
-	  case 'x':
+	  case 'x':		/* null byte */
 	  grow:
 	    while (len >= 10) {
 		rb_str_buf_cat(res, nul10, 10);
@@ -803,7 +893,7 @@ pack_pack(ary, fmt)
 	    rb_str_buf_cat(res, nul10, len);
 	    break;
 
-	  case 'X':
+	  case 'X':		/* back up byte */
 	  shrink:
 	    plen = RSTRING(res)->len;
 	    if (plen < len)
@@ -812,7 +902,7 @@ pack_pack(ary, fmt)
 	    RSTRING(res)->ptr[plen - len] = '\0';
 	    break;
 
-	  case '@':
+	  case '@':		/* null fill to absolute position */
 	    len -= RSTRING(res)->len;
 	    if (len > 0) goto grow;
 	    len = -len;
@@ -823,24 +913,24 @@ pack_pack(ary, fmt)
 	    rb_raise(rb_eArgError, "%% is not supported");
 	    break;
 
-	  case 'U':
+	  case 'U':		/* Unicode character */
 	    while (len-- > 0) {
-		unsigned long l;
+		long l;
 		char buf[8];
 		int le;
 
 		from = NEXTFROM;
 		if (NIL_P(from)) l = 0;
 		else {
-		    l = NUM2ULONG(from);
+		    l = NUM2UINT(from);
 		}
 		le = uv_to_utf8(buf, l);
 		rb_str_buf_cat(res, (char*)buf, le);
 	    }
 	    break;
 
-	  case 'u':
-	  case 'm':
+	  case 'u':		/* uuencoded string */
+	  case 'm':		/* base64 encoded string */
 	    from = NEXTFROM;
 	    StringValue(from);
 	    ptr = RSTRING(from)->ptr;
@@ -863,14 +953,14 @@ pack_pack(ary, fmt)
 	    }
 	    break;
 
-	  case 'M':
+	  case 'M':		/* quoted-printable encoded string */
 	    from = rb_obj_as_string(NEXTFROM);
 	    if (len <= 1)
 		len = 72;
 	    qpencode(res, from, len);
 	    break;
 
-	  case 'P':
+	  case 'P':		/* pointer to packed byte string */
 	    from = THISFROM;
 	    if (!NIL_P(from)) {
 		StringValue(from);
@@ -881,7 +971,7 @@ pack_pack(ary, fmt)
 	    }
 	    len = 1;
 	    /* FALL THROUGH */
-	  case 'p':
+	  case 'p':		/* pointer to string */
 	    while (len-- > 0) {
 		char *t;
 		from = NEXTFROM;
@@ -899,7 +989,7 @@ pack_pack(ary, fmt)
 	    }
 	    break;
 
-	  case 'w':
+	  case 'w':		/* BER compressed integer  */
 	    while (len-- > 0) {
 		unsigned long ul;
 		VALUE buf = rb_str_new(0, 0);
@@ -918,7 +1008,11 @@ pack_pack(ary, fmt)
 
 		if (NIL_P(from)) ul = 0;
 		else {
-		    ul = NUM2ULONG(from);
+		    long l = NUM2LONG(from);
+		    if (l < 0) {
+			rb_raise(rb_eArgError, "cannot compress negative numbers");
+		    }
+		    ul = l;
 		}
 
 		while (ul) {
@@ -1114,6 +1208,142 @@ infected_str_new(ptr, len, str)
     return s;
 }
     
+/*
+ *  call-seq:
+ *     str.unpack(format)   => anArray
+ *  
+ *  Decodes <i>str</i> (which may contain binary data) according to the
+ *  format string, returning an array of each value extracted. The
+ *  format string consists of a sequence of single-character directives,
+ *  summarized in the table at the end of this entry.
+ *  Each directive may be followed
+ *  by a number, indicating the number of times to repeat with this
+ *  directive. An asterisk (``<code>*</code>'') will use up all
+ *  remaining elements. The directives <code>sSiIlL</code> may each be
+ *  followed by an underscore (``<code>_</code>'') to use the underlying
+ *  platform's native size for the specified type; otherwise, it uses a
+ *  platform-independent consistent size. Spaces are ignored in the
+ *  format string. See also <code>Array#pack</code>.
+ *     
+ *     "abc \0\0abc \0\0".unpack('A6Z6')   #=> ["abc", "abc "]
+ *     "abc \0\0".unpack('a3a3')           #=> ["abc", " \000\000"]
+ *     "aa".unpack('b8B8')                 #=> ["10000110", "01100001"]
+ *     "aaa".unpack('h2H2c')               #=> ["16", "61", 97]
+ *     "\xfe\xff\xfe\xff".unpack('sS')     #=> [-2, 65534]
+ *     "now=20is".unpack('M*')             #=> ["now is"]
+ *     "whole".unpack('xax2aX2aX1aX2a')    #=> ["h", "e", "l", "l", "o"]
+ *
+ *  This table summarizes the various formats and the Ruby classes
+ *  returned by each.
+ *     
+ *     Format | Returns | Function
+ *     -------+---------+-----------------------------------------
+ *       A    | String  | with trailing nulls and spaces removed
+ *     -------+---------+-----------------------------------------
+ *       a    | String  | string
+ *     -------+---------+-----------------------------------------
+ *       B    | String  | extract bits from each character (msb first)
+ *     -------+---------+-----------------------------------------
+ *       b    | String  | extract bits from each character (lsb first)
+ *     -------+---------+-----------------------------------------
+ *       C    | Fixnum  | extract a character as an unsigned integer
+ *     -------+---------+-----------------------------------------
+ *       c    | Fixnum  | extract a character as an integer
+ *     -------+---------+-----------------------------------------
+ *       d,D  | Float   | treat sizeof(double) characters as
+ *            |         | a native double
+ *     -------+---------+-----------------------------------------
+ *       E    | Float   | treat sizeof(double) characters as
+ *            |         | a double in little-endian byte order
+ *     -------+---------+-----------------------------------------
+ *       e    | Float   | treat sizeof(float) characters as
+ *            |         | a float in little-endian byte order
+ *     -------+---------+-----------------------------------------
+ *       f,F  | Float   | treat sizeof(float) characters as
+ *            |         | a native float
+ *     -------+---------+-----------------------------------------
+ *       G    | Float   | treat sizeof(double) characters as
+ *            |         | a double in network byte order
+ *     -------+---------+-----------------------------------------
+ *       g    | Float   | treat sizeof(float) characters as a
+ *            |         | float in network byte order
+ *     -------+---------+-----------------------------------------
+ *       H    | String  | extract hex nibbles from each character
+ *            |         | (most significant first)
+ *     -------+---------+-----------------------------------------
+ *       h    | String  | extract hex nibbles from each character
+ *            |         | (least significant first)
+ *     -------+---------+-----------------------------------------
+ *       I    | Integer | treat sizeof(int) (modified by _)
+ *            |         | successive characters as an unsigned
+ *            |         | native integer
+ *     -------+---------+-----------------------------------------
+ *       i    | Integer | treat sizeof(int) (modified by _)
+ *            |         | successive characters as a signed
+ *            |         | native integer
+ *     -------+---------+-----------------------------------------
+ *       L    | Integer | treat four (modified by _) successive
+ *            |         | characters as an unsigned native
+ *            |         | long integer
+ *     -------+---------+-----------------------------------------
+ *       l    | Integer | treat four (modified by _) successive
+ *            |         | characters as a signed native
+ *            |         | long integer
+ *     -------+---------+-----------------------------------------
+ *       M    | String  | quoted-printable
+ *     -------+---------+-----------------------------------------
+ *       m    | String  | base64-encoded
+ *     -------+---------+-----------------------------------------
+ *       N    | Integer | treat four characters as an unsigned
+ *            |         | long in network byte order
+ *     -------+---------+-----------------------------------------
+ *       n    | Fixnum  | treat two characters as an unsigned
+ *            |         | short in network byte order
+ *     -------+---------+-----------------------------------------
+ *       P    | String  | treat sizeof(char *) characters as a
+ *            |         | pointer, and  return \emph{len} characters
+ *            |         | from the referenced location
+ *     -------+---------+-----------------------------------------
+ *       p    | String  | treat sizeof(char *) characters as a
+ *            |         | pointer to a  null-terminated string
+ *     -------+---------+-----------------------------------------
+ *       Q    | Integer | treat 8 characters as an unsigned 
+ *            |         | quad word (64 bits)
+ *     -------+---------+-----------------------------------------
+ *       q    | Integer | treat 8 characters as a signed 
+ *            |         | quad word (64 bits)
+ *     -------+---------+-----------------------------------------
+ *       S    | Fixnum  | treat two (different if _ used)
+ *            |         | successive characters as an unsigned
+ *            |         | short in native byte order
+ *     -------+---------+-----------------------------------------
+ *       s    | Fixnum  | Treat two (different if _ used) 
+ *            |         | successive characters as a signed short
+ *            |         | in native byte order
+ *     -------+---------+-----------------------------------------
+ *       U    | Integer | UTF-8 characters as unsigned integers
+ *     -------+---------+-----------------------------------------
+ *       u    | String  | UU-encoded
+ *     -------+---------+-----------------------------------------
+ *       V    | Fixnum  | treat four characters as an unsigned
+ *            |         | long in little-endian byte order
+ *     -------+---------+-----------------------------------------
+ *       v    | Fixnum  | treat two characters as an unsigned
+ *            |         | short in little-endian byte order
+ *     -------+---------+-----------------------------------------
+ *       w    | Integer | BER-compressed integer (see Array.pack)
+ *     -------+---------+-----------------------------------------
+ *       X    | ---     | skip backward one character
+ *     -------+---------+-----------------------------------------
+ *       x    | ---     | skip forward one character
+ *     -------+---------+-----------------------------------------
+ *       Z    | String  | with trailing nulls removed
+ *     -------+---------+-----------------------------------------
+ *       @    | ---     | skip to the offset given by the 
+ *            |         | length argument
+ *     -------+---------+-----------------------------------------
+ */
+
 static VALUE
 pack_unpack(str, fmt)
     VALUE str, fmt;
@@ -1417,7 +1647,7 @@ pack_unpack(str, fmt)
 	    PACK_LENGTH_ADJUST(unsigned short,2);
 	    while (len-- > 0) {
 		unsigned short tmp = 0;
-		memcpy(OFF16B(&tmp), s, NATINT_LEN(unsigned short,2));
+		memcpy(OFF16(&tmp), s, NATINT_LEN(unsigned short,2));
 		s += NATINT_LEN(unsigned short,2);
 		rb_ary_push(ary, UINT2NUM(ntohs(tmp)));
 	    }
@@ -1428,7 +1658,7 @@ pack_unpack(str, fmt)
 	    PACK_LENGTH_ADJUST(unsigned long,4);
 	    while (len-- > 0) {
 		unsigned long tmp = 0;
-		memcpy(OFF32B(&tmp), s, NATINT_LEN(unsigned long,4));
+		memcpy(OFF32(&tmp), s, NATINT_LEN(unsigned long,4));
 		s += NATINT_LEN(unsigned long,4);
 		rb_ary_push(ary, ULONG2NUM(ntohl(tmp)));
 	    }
@@ -1857,7 +2087,7 @@ uv_to_utf8(buf, uv)
 	buf[5] = (uv&0x3f)|0x80;
 	return 6;
     }
-    rb_raise(rb_eArgError, "uv_to_utf8(); too big value");
+    rb_raise(rb_eArgError, "pack(U): value out of range");
 }
 
 static const long utf8_limits[] = {
