@@ -2,8 +2,8 @@
 
   ruby.c -
 
-  $Author: matz $
-  $Date: 2003/07/30 07:08:55 $
+  $Author: nobu $
+  $Date: 2003/11/06 01:17:59 $
   created at: Tue Aug 10 12:47:31 JST 1993
 
   Copyright (C) 1993-2003 Yukihiro Matsumoto
@@ -429,6 +429,7 @@ proc_options(argc, argv)
     char *argv0 = argv[0];
     int do_search;
     char *s;
+    NODE *volatile script_node = 0;
 
     int version = 0;
     int copyright = 0;
@@ -751,6 +752,8 @@ proc_options(argc, argv)
 		    script = dln_find_file(argv[0], getenv(PATH_ENV));
 		}
 		if (!script) script = argv[0];
+		script = ruby_sourcefile = rb_source_filename(script);
+		script_node = NEW_NEWLINE(0);
 	    }
 #ifdef DOSISH
 	    translate_char(script, '\\', '/');
