@@ -2,11 +2,11 @@
 
   rubyio.h -
 
-  $Author: nobu $
-  $Date: 2001/12/17 08:13:17 $
+  $Author: matz $
+  $Date: 2003/04/14 09:04:43 $
   created at: Fri Nov 12 16:47:09 JST 1993
 
-  Copyright (C) 1993-2000 Yukihiro Matsumoto
+  Copyright (C) 1993-2003 Yukihiro Matsumoto
 
 **********************************************************************/
 
@@ -23,7 +23,7 @@ typedef struct OpenFile {
     int pid;			/* child's pid (for pipes) */
     int lineno;			/* number of lines read */
     char *path;			/* pathname for file */
-    void (*finalize)();		/* finalize proc */
+    void (*finalize) _((struct OpenFile*,int)); /* finalize proc */
 } OpenFile;
 
 #define FMODE_READABLE  1
@@ -31,10 +31,17 @@ typedef struct OpenFile {
 #define FMODE_READWRITE 3
 #define FMODE_BINMODE   4
 #define FMODE_SYNC      8
+#define FMODE_WBUF     16
+#define FMODE_RBUF     32
 
 #define GetOpenFile(obj,fp) rb_io_check_closed((fp) = RFILE(rb_io_taint_check(obj))->fptr)
 
 #define MakeOpenFile(obj, fp) do {\
+    if (RFILE(obj)->fptr) {\
+	rb_io_close(obj);\
+	free(RFILE(obj)->fptr);\
+	RFILE(obj)->fptr = 0;\
+    }\
     fp = 0;\
     fp = RFILE(obj)->fptr = ALLOC(OpenFile);\
     fp->f = fp->f2 = NULL;\
@@ -51,12 +58,16 @@ typedef struct OpenFile {
 FILE *rb_fopen _((const char*, const char*));
 FILE *rb_fdopen _((int, const char*));
 int rb_getc _((FILE*));
+long rb_io_fread _((char *, long, FILE *));
+long rb_io_fwrite _((const char *, long, FILE *));
 int  rb_io_mode_flags _((const char*));
 void rb_io_check_writable _((OpenFile*));
 void rb_io_check_readable _((OpenFile*));
 void rb_io_fptr_finalize _((OpenFile*));
 void rb_io_synchronized _((OpenFile*));
 void rb_io_check_closed _((OpenFile*));
+int rb_io_wait_readable _((int));
+int rb_io_wait_writable _((int));
 
 VALUE rb_io_taint_check _((VALUE));
 void rb_eof_error _((void));
