@@ -9,11 +9,11 @@ while gets()
   send = $_.rindex("\n%%") + 1
   $_ = $_[sbeg, send-sbeg]
   sub!(/.*\n/, "")
-  gsub!(/'{'/, "'\001'")
+  gsub!(/'\{'/, "'\001'")
   gsub!(/'}'/, "'\002'")
-  gsub!('\*/', "\003\003")
-  gsub!("/\\*[^\003]*\003\003", '')
-  while gsub!(/{[^}{]*}/, ''); end
+  gsub!(%r{\*/}, "\003\003")
+  gsub!(%r{/\*[^\003]*\003\003}, '')
+  while gsub!(/\{[^{}]*}/, ''); end
   gsub!(/'\001'/, "'{'")
   gsub!(/'\002'/, "'}'")
   while gsub!(/^[ \t]*\n(\s)/, '\1'); end
