@@ -2,8 +2,8 @@
 
   struct.c -
 
-  $Author: usa $
-  $Date: 2003/07/12 08:02:26 $
+  $Author: matz $
+  $Date: 2003/11/04 09:13:57 $
   created at: Tue Mar 22 18:44:30 JST 1995
 
   Copyright (C) 1993-2003 Yukihiro Matsumoto
@@ -544,10 +544,6 @@ rb_struct_select(argc, argv, s)
     VALUE result;
     long i;
 
-    if (!rb_block_given_p()) {
-	rb_warn("Struct#select(index..) is deprecated; use Struct#values_at");
-	return rb_struct_values_at(argc, argv, s);
-    }
     if (argc > 0) {
 	rb_raise(rb_eArgError, "wrong number arguments(%d for 0)", argc);
     }
