@@ -3,7 +3,7 @@
   time.c -
 
   $Author: matz $
-  $Date: 2003/07/24 18:33:40 $
+  $Date: 2003/11/28 14:23:31 $
   created at: Tue Dec 28 14:31:59 JST 1993
 
   Copyright (C) 1993-2003 Yukihiro Matsumoto
@@ -51,32 +51,47 @@ time_s_alloc(klass)
 
     obj = Data_Make_Struct(klass, struct time_object, 0, time_free, tobj);
     tobj->tm_got=0;
-    if (gettimeofday(&tobj->tv, 0) < 0) {
-	rb_sys_fail("gettimeofday");
-    }
+    tobj->tv.tv_sec = 0;
+    tobj->tv.tv_usec = 0;
 
     return obj;
 }
 
-static VALUE
-time_s_now(klass)
-    VALUE klass;
+static void
+time_modify(time)
+    VALUE time;
 {
-    return rb_obj_alloc(klass);
+    rb_check_frozen(time);
+    if (!OBJ_TAINTED(time) && rb_safe_level() >= 4)
+	rb_raise(rb_eSecurityError, "Insecure: can't modify Time");
 }
 
+static VALUE
+time_init(time)
+    VALUE time;
+{
+    struct time_object *tobj;
+
+    time_modify(time);
+    GetTimeval(time, tobj);
+    tobj->tm_got=0;
+    tobj->tv.tv_sec = 0;
+    tobj->tv.tv_usec = 0;
+    if (gettimeofday(&tobj->tv, 0) < 0) {
+	rb_sys_fail("gettimeofday");
+    }
+
+    return time;
+}
 
 #define NDIV(x,y) (-(-((x)+1)/(y))-1)
 #define NMOD(x,y) ((y)-(-((x)+1)%(y))-1)
 
-static VALUE
-time_new_internal(klass, sec, usec)
-    VALUE klass;
+void
+time_overflow_p(sec, usec)
     time_t sec, usec;
 {
-    VALUE obj;
     time_t tmp;
-    struct time_object *tobj;
 
     if (usec >= 1000000) {	/* usec positive overflow */
 	tmp = sec + usec / 1000000;
@@ -98,13 +113,22 @@ time_new_internal(klass, sec, usec)
     if (sec < 0 || (sec == 0 && usec < 0))
 	rb_raise(rb_eArgError, "time must be positive");
 #endif
+}
 
-    obj = Data_Make_Struct(klass, struct time_object, 0, time_free, tobj);
-    tobj->tm_got = 0;
+static VALUE
+time_new_internal(klass, sec, usec)
+    VALUE klass;
+    time_t sec, usec;
+{
+    VALUE time = time_s_alloc(klass);
+    struct time_object *tobj;
+
+    GetTimeval(time, tobj);
+    time_overflow_p(sec, usec);
     tobj->tv.tv_sec = sec;
     tobj->tv.tv_usec = usec;
 
-    return obj;
+    return time;
 }
 
 VALUE
@@ -757,15 +781,6 @@ time_hash(time)
     return LONG2FIX(hash);
 }
 
-static void
-time_modify(time)
-    VALUE time;
-{
-    rb_check_frozen(time);
-    if (!OBJ_TAINTED(time) && rb_safe_level() >= 4)
-	rb_raise(rb_eSecurityError, "Insecure: can't modify Time");
-}
-
 static VALUE
 time_init_copy(copy, time)
     VALUE copy, time;
@@ -994,6 +1009,16 @@ time_minus(time1, time2)
     return time2;
 }
 
+static VALUE
+time_succ(time)
+    VALUE time;
+{
+    struct time_object *tobj;
+
+    GetTimeval(time, tobj);
+    return rb_time_new(tobj->tv.tv_sec + 1, tobj->tv.tv_usec);
+}
+
 static VALUE
 time_sec(time)
     VALUE time;
@@ -1293,20 +1318,16 @@ time_s_times(obj)
 }
 
 static VALUE
-time_dump(argc, argv, time)
-    int argc;
-    VALUE *argv;
+time_mdump(time)
     VALUE time;
 {
-    VALUE dummy;
     struct time_object *tobj;
     struct tm *tm;
     unsigned long p, s;
-    unsigned char buf[8];
+    char buf[8];
     time_t t;
     int i;
 
-    rb_scan_args(argc, argv, "01", &dummy);
     GetTimeval(time, tobj);
 
     t = tobj->tv.tv_sec;
@@ -1337,15 +1358,35 @@ time_dump(argc, argv, time)
 }
 
 static VALUE
-time_load(klass, str)
-    VALUE klass, str;
+time_dump(argc, argv, time)
+    int argc;
+    VALUE *argv;
+    VALUE time;
 {
+    VALUE str;
+
+    rb_scan_args(argc, argv, "01", 0);
+    str = time_mdump(time); 
+    if (FL_TEST(time, FL_EXIVAR)) {
+	rb_copy_generic_ivar(str, time);
+	FL_SET(str, FL_EXIVAR);
+    }
+
+    return str;
+}
+
+static VALUE
+time_mload(time, str)
+    VALUE time, str;
+{
+    struct time_object *tobj;
     unsigned long p, s;
     time_t sec, usec;
     unsigned char *buf;
     struct tm tm;
     int i;
 
+    time_modify(time);
     StringValue(str);
     buf = (unsigned char *)RSTRING(str)->ptr;
     if (RSTRING(str)->len != 8) {
@@ -1361,21 +1402,43 @@ time_load(klass, str)
     }
 
     if ((p & (1<<31)) == 0) {
-	return time_new_internal(klass, p, s);
+	sec = p;
+	usec = s;
     }
-    p &= ~(1<<31);
-    tm.tm_year = (p >> 14) & 0x1ffff;
-    tm.tm_mon  = (p >> 10) & 0xf;
-    tm.tm_mday = (p >>  5) & 0x1f;
-    tm.tm_hour =  p        & 0x1f;
-    tm.tm_min  = (s >> 26) & 0x3f;
-    tm.tm_sec  = (s >> 20) & 0x3f;
-    tm.tm_isdst = 0;
+    else {
+	p &= ~(1<<31);
+	tm.tm_year = (p >> 14) & 0x1ffff;
+	tm.tm_mon  = (p >> 10) & 0xf;
+	tm.tm_mday = (p >>  5) & 0x1f;
+	tm.tm_hour =  p        & 0x1f;
+	tm.tm_min  = (s >> 26) & 0x3f;
+	tm.tm_sec  = (s >> 20) & 0x3f;
+	tm.tm_isdst = 0;
 
-    sec = make_time_t(&tm, Qtrue);
-    usec = (time_t)(s & 0xfffff);
+	sec = make_time_t(&tm, Qtrue);
+	usec = (time_t)(s & 0xfffff);
+    }
+    time_overflow_p(sec, usec);
 
-    return time_new_internal(klass, sec, usec);
+    GetTimeval(time, tobj);
+    tobj->tm_got = 0;
+    tobj->tv.tv_sec = sec;
+    tobj->tv.tv_usec = usec;
+    return time;
+}
+
+static VALUE
+time_load(klass, str)
+    VALUE klass, str;
+{
+    VALUE time = time_s_alloc(klass);
+
+    if (FL_TEST(str, FL_EXIVAR)) {
+	rb_copy_generic_ivar(time, str);
+	FL_SET(time, FL_EXIVAR);
+    }
+    time_mload(time, str);
+    return time;
 }
 
 void
@@ -1384,8 +1447,8 @@ Init_Time()
     rb_cTime = rb_define_class("Time", rb_cObject);
     rb_include_module(rb_cTime, rb_mComparable);
 
-    rb_define_singleton_method(rb_cTime, "now", time_s_now, 0);
     rb_define_alloc_func(rb_cTime, time_s_alloc);
+    rb_define_singleton_method(rb_cTime, "now", rb_class_new_instance, -1);
     rb_define_singleton_method(rb_cTime, "at", time_s_at, -1);
     rb_define_singleton_method(rb_cTime, "utc", time_s_mkutc, -1);
     rb_define_singleton_method(rb_cTime, "gm", time_s_mkutc, -1);
@@ -1399,6 +1462,7 @@ Init_Time()
     rb_define_method(rb_cTime, "<=>", time_cmp, 1);
     rb_define_method(rb_cTime, "eql?", time_eql, 1);
     rb_define_method(rb_cTime, "hash", time_hash, 0);
+    rb_define_method(rb_cTime, "initialize", time_init, 0);
     rb_define_method(rb_cTime, "initialize_copy", time_init_copy, 1);
 
     rb_define_method(rb_cTime, "localtime", time_localtime, 0);
@@ -1417,6 +1481,7 @@ Init_Time()
     rb_define_method(rb_cTime, "+", time_plus, 1);
     rb_define_method(rb_cTime, "-", time_minus, 1);
 
+    rb_define_method(rb_cTime, "succ", time_succ, 0);
     rb_define_method(rb_cTime, "sec", time_sec, 0);
     rb_define_method(rb_cTime, "min", time_min, 0);
     rb_define_method(rb_cTime, "hour", time_hour, 0);
@@ -1446,4 +1511,9 @@ Init_Time()
     /* methods for marshaling */
     rb_define_method(rb_cTime, "_dump", time_dump, -1);
     rb_define_singleton_method(rb_cTime, "_load", time_load, 1);
+#if 0
+    /* Time will support marshal_dump and marshal_load in the future (1.9 maybe) */
+    rb_define_method(rb_cTime, "marshal_dump", time_mdump, 0);
+    rb_define_method(rb_cTime, "marshal_load", time_mload, 1);
+#endif
 }
