@@ -3,7 +3,7 @@
   util.c -
 
   $Author: matz $
-  $Date: 2003/07/11 07:10:14 $
+  $Date: 2003/11/22 03:59:18 $
   created at: Fri Mar 10 17:22:34 JST 1995
 
   Copyright (C) 1993-2003 Yukihiro Matsumoto
@@ -636,14 +636,25 @@ ruby_strdup(str)
 char *
 ruby_getcwd()
 {
+#ifdef HAVE_GETCWD
     int size = 200;
     char *buf = xmalloc(size);
 
     while (!getcwd(buf, size)) {
-	if (errno != ERANGE) rb_sys_fail(0);
+	if (errno != ERANGE) rb_sys_fail("getcwd");
 	size *= 2;
 	buf = xrealloc(buf, size);
     }
+#else
+# ifndef PATH_MAX
+#  define PATH_MAX 8192
+# endif
+    char *buf = xmalloc(PATH_MAX+1);
+
+    if (!getwd(buf)) {
+	rb_sys_fail("getwd");
+    }
+#endif
     return buf;
 }
 
@@ -757,6 +768,13 @@ ruby_strtod(string, endPtr)
 	sign = FALSE;
     }
 
+    /* skip preceding zeros */
+    if (*p == '0') {
+	while (*p == '0')
+	    p++;
+	p--;
+    }
+
     /*
      * Count the number of digits in the mantissa (including the decimal
      * point), and also locate the decimal point.
